package cn.bs.bsmanagebasic.aspect;

import cn.bs.bsmanagebasic.common.util.CacheTool;
import cn.bs.bsmanagebasic.entity.User;
import cn.bs.bsmanagecommon.exception.APIBusinessException;
import cn.bs.bsmanagecommon.exception.BusinessException;
import cn.bs.bsmanagecommon.result.ResultBean;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

/**
 * @author sunjj
 * @version 1.0.0
 * @ClassName ControllerAspect.java
 * @Description 针对Conroller层的AOP拦截
 * @createTime 2019年07月22日 12:27:00
 */
@Component
@Aspect
@Slf4j
public class ControllerAspect {

    @Around("execution(* cn.bs.bsmanagebasic.controller.*.*(..))")
    public Object around(ProceedingJoinPoint pjp){
        long startTime = System.currentTimeMillis();
        ResultBean<?> result;
        try {
            User user = isLogin();
            result = (ResultBean<?>) pjp.proceed();
            log.info(pjp.getSignature().getName() + "  use time:" + (System.currentTimeMillis() - startTime));
        } catch (Throwable e) {
            result = handlerException(pjp, e);
        }
        return result;
    }

    private User isLogin(){
        ServletRequestAttributes sra = (ServletRequestAttributes)RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = sra.getRequest();
        String url = request.getRequestURL().toString();
        if(url.indexOf("/user/login")>0 ||
                url.indexOf("/pickupQueue/insert")>0 ||
                url.indexOf("/adlogs")>0 ||
                url.indexOf("/delivery/getPickupCodeByPhone")>0){
            return null;
        }
        HttpSession session= request.getSession();
        User user = JSON.parseObject(session.getAttribute(session.getId()).toString(),User.class);
        if(user == null){
            throw new BusinessException("未登录，请重新登录");
        }
        return user;
    }

    @Around("execution(* cn.bs.bsmanagebasic.controller.*.*(..))")
    public Object around2(ProceedingJoinPoint pjp){
        return this.around(pjp);
    }

    private ResultBean<?> handlerException(ProceedingJoinPoint pjp, Throwable e) {
        ResultBean<?> result = new ResultBean();

        // 已知异常
        if (e instanceof APIBusinessException){
            result.setMsg(e.getMessage());
            result.setCode(ResultBean.FAIL);
        } else if(e instanceof BusinessException){
            log.error(pjp.getSignature() + " error ", e);
            result.setMsg(e.toString());
            result.setCode(ResultBean.FAIL);
        } else if(e instanceof Exception){
            log.error(pjp.getSignature() + " error ", e);
            result.setMsg(e.toString());
            result.setCode(ResultBean.NO_LOGIN);
        } else {
            log.error(pjp.getSignature() + " error ", e);
            result.setMsg("系统错误");
            result.setCode(ResultBean.NO_LOGIN);
        }
        return result;
    }
}
