package cn.bs.bsmanagebasic.config;

import cn.bs.bsmanagebasic.common.util.redis.JedisClient;
import cn.bs.bsmanagebasic.common.util.redis.JedisClientSingle;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import redis.clients.jedis.JedisPool;
import redis.clients.jedis.JedisPoolConfig;

/**
 * @author taohaijun
 * @Description:
 * @date 2018/11/19 10:25
 */
@Configuration
@EnableConfigurationProperties(JedisProperties.class)
@ConditionalOnClass(JedisClient.class)
@Slf4j
public class JedisConfig {

    @Autowired
    private JedisProperties prop;

    @Value("${spring.redis.pool.maxTotal}")
    private int maxTotal;
    @Value("${spring.redis.pool.min-idle}")
    private int maxIdle;
    @Value("${spring.redis.pool.maxWaitMillis}")
    private int maxWaitMillis;

    @Bean(name = "jedisPool")
    public JedisPool jedisPool() {
        JedisPoolConfig config = new JedisPoolConfig();
        config.setMaxTotal(maxTotal);
        config.setMaxIdle(maxIdle);
        config.setMaxWaitMillis(maxWaitMillis);
        return new JedisPool(config, prop.getHost(), prop.getPort(), prop.getTimeout(), prop.getPassword(), 0);
    }

    @Bean(name = "jedisClient")
    @ConditionalOnMissingBean(JedisClient.class)
    public JedisClient jedisClient(@Qualifier("jedisPool") JedisPool pool) {
        log.info("初始化……Redis Client==Host={},Port={}", prop.getHost(), prop.getPort());
        JedisClient redisClient = new JedisClientSingle(pool);
        return redisClient;
    }
}
