package cn.bs.bsmanagebasic.config;

import com.mysql.jdbc.jdbc2.optional.MysqlXADataSource;
import org.apache.ibatis.session.SqlSessionFactory;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.mybatis.spring.annotation.MapperScan;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.jta.atomikos.AtomikosDataSourceBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;

import javax.sql.DataSource;
import java.sql.SQLException;

/**
 *
 */
@Configuration
@MapperScan(basePackages = JeesiteDatasourceConfig.PACKAGE, sqlSessionFactoryRef = "jeesiteSqlSessionFactory")
public class JeesiteDatasourceConfig {

    static final String PACKAGE = "cn.bs.bsmanagebasic.dao.jeesite";

    private static final String DATA_SOURCE = "jeesiteDataSource";
    private static final String SESSION_FACTORY = "jeesiteSqlSessionFactory";
    private static final String XA_DATA_SOURCE = "xaJeeSiteDataSource";
    private static final String XA_PROPERTIES = "spring.datasource.xa";
    private static final String DATA_SOURCE_PROPERTIES = "spring.datasource.xa.jeesite-xa-properties";

    /*
    @Bean(name = "jeesiteDataSource")
    @Primary
    @ConfigurationProperties(prefix = "spring.datasource.hikari.jeesite")
    public HikariDataSource dataSource() {
        return new HikariDataSource();
    }

    @Bean(name = "jeesiteTransactionManager")
    @Primary
    public DataSourceTransactionManager transactionManager() {
        return new DataSourceTransactionManager(this.dataSource());
    }
    */

    @Bean(name = DATA_SOURCE)
    @ConfigurationProperties(prefix = DATA_SOURCE_PROPERTIES)
    public MysqlXADataSource dataSource() throws SQLException {
        MysqlXADataSource xaDataSource = new MysqlXADataSource();
        xaDataSource.setPinGlobalTxToPhysicalConnection(true);
        xaDataSource.setPinGlobalTxToPhysicalConnection(true);
        return xaDataSource;
    }

    @Bean(name = XA_DATA_SOURCE)
    @Primary
    @ConfigurationProperties(prefix = XA_PROPERTIES)
    public AtomikosDataSourceBean jeeSiteAtomikosDataSource(@Qualifier(DATA_SOURCE) MysqlXADataSource dataSource) throws SQLException {
        AtomikosDataSourceBean atomikosDataSourceBean = new AtomikosDataSourceBean();
        atomikosDataSourceBean.setXaDataSource(dataSource);
        atomikosDataSourceBean.setUniqueResourceName(XA_DATA_SOURCE);
        return atomikosDataSourceBean;
    }

    @Bean(name = SESSION_FACTORY)
    @Primary
    public SqlSessionFactory sqlSessionFactory(@Qualifier(XA_DATA_SOURCE) DataSource dataSource) throws Exception {
        final SqlSessionFactoryBean sessionFactory = new SqlSessionFactoryBean();
        sessionFactory.setDataSource(dataSource);
        // sessionFactory.getObject().getConfiguration().setMapUnderscoreToCamelCase(true);
        sessionFactory.setMapperLocations(new PathMatchingResourcePatternResolver().getResources("classpath:mapper/jeesite/*.xml"));
        return sessionFactory.getObject();
    }


}
