package cn.bs.bsmanagebasic.config;

import com.mysql.jdbc.jdbc2.optional.MysqlXADataSource;
import org.apache.ibatis.session.SqlSessionFactory;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.mybatis.spring.annotation.MapperScan;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.jta.atomikos.AtomikosDataSourceBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.jdbc.core.JdbcTemplate;

import javax.sql.DataSource;
import java.sql.SQLException;

@Configuration
@MapperScan(basePackages = JisuDatasourceConfig.PACKAGE, sqlSessionFactoryRef = "jisuSqlSessionFactory")
public class JisuDatasourceConfig {

    static final String PACKAGE = "cn.bs.bsmanagebasic.dao.jisu";

    private static final String DATA_SOURCE = "jisuDataSource";
    private static final String SESSION_FACTORY = "jisuSqlSessionFactory";
    private static final String XA_DATA_SOURCE = "xaJisuDataSource";
    private static final String XA_PROPERTIES = "spring.datasource.xa";
    private static final String DATA_SOURCE_PROPERTIES = "spring.datasource.xa.jisu-xa-properties";

    /*
    @Bean(name = "jisuDataSource")
    @ConfigurationProperties(prefix = "spring.datasource.hikari.jisu")
    public HikariDataSource dataSource() {
        return DataSourceBuilder.create().type(HikariDataSource.class).build();
    }

    @Bean(name = "jisuTransactionManager")
    public DataSourceTransactionManager transactionManager() {
        return new DataSourceTransactionManager(this.dataSource());
    }
    */

    @Bean(name = DATA_SOURCE)
    @ConfigurationProperties(prefix = DATA_SOURCE_PROPERTIES)
    public MysqlXADataSource dataSource() throws SQLException {
        MysqlXADataSource xaDataSource = new MysqlXADataSource();
        xaDataSource.setPinGlobalTxToPhysicalConnection(true);
        xaDataSource.setPinGlobalTxToPhysicalConnection(true);
        return xaDataSource;
    }

    @Bean(name = XA_DATA_SOURCE)
    @ConfigurationProperties(prefix = XA_PROPERTIES)
    public AtomikosDataSourceBean jiSuAtomikosDataSource(@Qualifier(DATA_SOURCE) MysqlXADataSource dataSource) throws SQLException {
        AtomikosDataSourceBean atomikosDataSourceBean = new AtomikosDataSourceBean();
        atomikosDataSourceBean.setXaDataSource(dataSource);
        atomikosDataSourceBean.setUniqueResourceName("xaJisuSite");
        return atomikosDataSourceBean;
    }

    @Bean(name = SESSION_FACTORY)
    public SqlSessionFactory sqlSessionFactory(@Qualifier(XA_DATA_SOURCE) DataSource dataSource) throws Exception {
        final SqlSessionFactoryBean sessionFactory = new SqlSessionFactoryBean();
        sessionFactory.setDataSource(dataSource);
        // sessionFactory.getObject().getConfiguration().setMapUnderscoreToCamelCase(true);
        sessionFactory.setMapperLocations(new PathMatchingResourcePatternResolver().getResources("classpath:/mapper/jisu/*.xml"));
        return sessionFactory.getObject();
    }

    @Bean(name = "jisuJdbcTemplate")
    public JdbcTemplate primaryJdbcTemplate(@Qualifier(DATA_SOURCE) DataSource dataSource) {
        return new JdbcTemplate(dataSource);
    }


}