package cn.bs.bsmanagebasic.config;

import lombok.Data;
import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.BindingBuilder;
import org.springframework.amqp.core.DirectExchange;
import org.springframework.amqp.core.Queue;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Component;

/**
 * 〈功能概述〉<br>
 *
 * @className: Rabbitconfig
 * @package: cn.bs.bsmanagebasic.utils
 * @author: kael
 * @date: 2019/11/11 0011 下午 14:44
 */
@Data
@Component
@ConfigurationProperties(prefix = "spring.rabbitmq")
@Configuration
public class RabbitConfig {

    /**
     * Express
     */
    private String expressQueue;
    private String expressExchange;
    private String expressKey;
    /**
     * 极光推送
     */
    private String jPushQueueName;
    private String jPushExchangeName;
    private String jPushRoutingKey;


    /**
     * 定义ES删除队列
     * @return
     */
    @Bean
    public Queue expressQueue() {
        return new Queue(expressQueue,true);
    }

    /**
     * 定义ES删除交换机
     * @return
     */
    @Bean
    DirectExchange expressExchange() {
        return new DirectExchange(expressExchange,true,false);
    }

    /**
     * ES删除队列与交换机绑定
     * @param expressQueue
     * @param expressExchange
     * @return
     */
    @Bean
    public Binding bindingESDeleteExchange(Queue expressQueue, DirectExchange expressExchange) {
        return BindingBuilder.bind(expressQueue).to(expressExchange).with(expressKey);
    }

    /**
     * 定义极光推送队列
     * @return
     */
    @Bean
    public Queue jPushQueue() {
        return new Queue(jPushQueueName,true);
    }

    /**
     * 定义极光推送交换机
     * @return
     */
    @Bean
    DirectExchange jPushExchange() {
        return new DirectExchange(jPushExchangeName,true,false);
    }

    /**
     * 极光推送队列与交换机绑定
     * @param jPushQueue
     * @param jPushExchange
     * @return
     */
    @Bean
    public Binding bindingjPushExchange(Queue jPushQueue, DirectExchange jPushExchange) {
        return BindingBuilder.bind(jPushQueue).to(jPushExchange).with(jPushRoutingKey);
    }
}
