package cn.bs.bsmanagebasic.controller;

import cn.bs.bsmanagebasic.common.constants.ChooseOptionsEnum;
import cn.bs.bsmanagebasic.entity.*;
import cn.bs.bsmanagebasic.service.*;
import cn.bs.bsmanagecommon.result.ResultBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author sunjj
 * @version 1.0.0
 * @ClassName CommonController.java
 * @Description BSManage options请求
 * @createTime 2019年07月22日 15:43:00
 */
@RestController
@RequestMapping("/options")
@CrossOrigin(allowCredentials ="true")
public class CommonController {
    @Autowired
    private AddressCityService addressCityService;
    @Autowired
    private AddressProvinceService addressProvinceService;
    @Autowired
    private AddressDistrictService addressDistrictService;
    @Autowired
    private AddressTownService addressTownService;
    @Autowired
    private ChooseoptionService chooseoptionService;

    /**
     * @description 获取省份列表
     * @author sunjj
     */
    @GetMapping("/getProvinces")
    public ResultBean<List<AddressProvince>> getProvinces() {
        return new ResultBean<List<AddressProvince>>(addressProvinceService.getProvinces());
    }

    /**
     * 根据省份Id获取城市列表
     */
    @GetMapping(value = {"/getCities/{provinceId}", "/getCities"})
    public ResultBean<List<AddressCity>> getCities(@PathVariable(required = false) Integer provinceId) {
        return new ResultBean<List<AddressCity>>(addressCityService.getCities(provinceId));
    }

    /**
     * 根据城市Id获取地区列表
     */
    @GetMapping(value = {"/getDistricts/{cityId}", "/getDistricts"})
    public ResultBean<List<AddressDistrict>> getDistricts(@PathVariable(required = false) Long cityId) {
        return new ResultBean<List<AddressDistrict>>(addressDistrictService.getDistricts(cityId));
    }

    /**
     * 根据地区Id获取镇列表信息
     */
    @GetMapping(value = {"/getTownships/{areaName}", "/getTownships"})
    public ResultBean<List<AddressTown>> getTownships(@PathVariable(required = false) Long areaName) {
        return new ResultBean<List<AddressTown>>(addressTownService.getTownships(areaName));
    }

    /**
     * @description 获取门店类型
     * @author sunjj
     */
    @GetMapping("/getStoreType")
    public ResultBean<List<TBsChooseoption>> getStoreType() {
        return new ResultBean<List<TBsChooseoption>>(
                chooseoptionService.getOptionsByCode(ChooseOptionsEnum.STORE_TYPE.getCode()));
    }

    /**
     * @description 获取短信通知类型
     * @author sunjj
     */
    @RequestMapping("/getReminderType")
    public ResultBean<List<TBsChooseoption>> getReminderType() {
        return new ResultBean<List<TBsChooseoption>>(
                chooseoptionService.getOptionsByCode(ChooseOptionsEnum.REMINDER_TYPE.getCode()));
    }
}
