package cn.bs.bsmanagebasic.controller;

import cn.bs.bsmanagebasic.common.util.Pager;
import cn.bs.bsmanagebasic.entity.ExpressDept;
import cn.bs.bsmanagebasic.entity.ExpressDeptAccount;
import cn.bs.bsmanagebasic.entity.dto.ExpressAccountConsumeRecordDto;
import cn.bs.bsmanagebasic.entity.dto.ExpressAccountRechargeRecordDto;
import cn.bs.bsmanagebasic.entity.filter.ExpressDeptFilter;
import cn.bs.bsmanagebasic.service.ExpressDeptService;
import cn.bs.bsmanagecommon.requst.RequestBean;
import cn.bs.bsmanagecommon.result.ResultBean;
import com.alibaba.fastjson.JSON;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author sunjj
 * @version 1.0.0
 * @ClassName ExpressDeptController.java
 * @Description 快递公司分部管理
 * @createTime 2019年11月18日 09:49:00
 */
@RestController
@RequestMapping("/expressDept")
@CrossOrigin
public class ExpressDeptController {

    @Autowired
    private ExpressDeptService expressDeptService;

    @PostMapping("/addDept")
    public ResultBean addDept(@RequestBody String string) {
        expressDeptService.addDept(JSON.parseObject(JSON.parseObject(string, RequestBean.class).getData(), ExpressDept.class));
        return new ResultBean();
    }

    @PostMapping("/getDepts")
    public ResultBean<Pager<ExpressDept>> getDepts(@RequestBody String string) {
        Pager<ExpressDept> page = expressDeptService.getDepts(JSON.parseObject(JSON.parseObject(string, RequestBean.class).getData(), ExpressDeptFilter.class));
        return new ResultBean(page);
    }

    @PostMapping("/getDeptDetail")
    public ResultBean<ExpressDept> getDeptDetail(@RequestBody String string) {
        ExpressDept dept = expressDeptService.getDeptDetail(JSON.parseObject(JSON.parseObject(string, RequestBean.class).getData(), ExpressDeptFilter.class));
        return new ResultBean<>(dept);
    }

    @PostMapping("/editDept")
    public ResultBean editDept(@RequestBody String string) {
        expressDeptService.editDept(JSON.parseObject(JSON.parseObject(string, RequestBean.class).getData(), ExpressDept.class));
        return new ResultBean();
    }

    @PostMapping("/addDeptAccount")
    public ResultBean addDeptAccount(@RequestBody String string) {
        expressDeptService.addDeptAccount(JSON.parseObject(JSON.parseObject(string, RequestBean.class).getData(), ExpressDeptAccount.class));
        return new ResultBean();
    }

    @PostMapping("/getDeptAccounts")
    public ResultBean<Pager<ExpressDeptAccount>> getDeptAccounts(@RequestBody String string) {
        Pager<ExpressDeptAccount> page = expressDeptService.getDeptAccounts(JSON.parseObject(JSON.parseObject(string, RequestBean.class).getData(), ExpressDeptFilter.class));
        return new ResultBean(page);
    }

    @PostMapping("/editDeptAccount")
    public ResultBean editDeptAccount(@RequestBody String string) {
        expressDeptService.editDeptAccount(JSON.parseObject(JSON.parseObject(string, RequestBean.class).getData(), ExpressDeptAccount.class));
        return new ResultBean();
    }

    @GetMapping(value = {"/getDeptsOption/{expId}", "/getDeptsOption"})
    public ResultBean<List<ExpressDept>> getDeptsForOption(@PathVariable(required = false) Long expId) {
        List<ExpressDept> depts = expressDeptService.getDeptsForOption(expId);
        return new ResultBean<>(depts);
    }

    @PostMapping("/accountRecharge")
    public ResultBean expressAccountRecharge(@RequestBody String string) {
        expressDeptService.expressAccountRecharge(JSON.parseObject(JSON.parseObject(string, RequestBean.class).getData(), ExpressDeptFilter.class));
        return new ResultBean();
    }

    @PostMapping("/getRecords")
    public ResultBean<Pager<ExpressAccountRechargeRecordDto>> getRecords(@RequestBody String string) {
        Pager<ExpressAccountRechargeRecordDto> page = expressDeptService.getRecords(JSON.parseObject(JSON.parseObject(string, RequestBean.class).getData(), ExpressDeptFilter.class));
        return new ResultBean(page);
    }


    /**
     * 通过id删除单条数据
     *
     * @param id
     * @return
     */
    @RequestMapping(value = "/delete/{id}", method = {RequestMethod.GET})
    public ResultBean<Boolean> deleteById(@PathVariable Long id) {
        return new ResultBean<Boolean>(expressDeptService.deleteById(id));
    }

    /**
     * 保存dept
     *
     * @param data
     * @return
     */
    @RequestMapping(value = "/save", method = {RequestMethod.POST})
    public ResultBean<Boolean> save(@RequestBody String data) {
        return new ResultBean<Boolean>(expressDeptService.save(JSON.parseObject(JSON.parseObject(data, RequestBean.class).getData(), ExpressDept.class)));
    }

    /**
     * 保存deptAcount
     *
     * @param data
     * @return
     */
    @RequestMapping(value = "/saveAcount", method = {RequestMethod.POST})
    public ResultBean<Boolean> saveAcount(@RequestBody String data) {
        return new ResultBean<Boolean>(expressDeptService.saveAcount(JSON.parseObject(JSON.parseObject(data, RequestBean.class).getData(), ExpressDeptAccount.class)));
    }

    /**
     * 通过id启用
     *
     * @param id
     * @return
     */
    @RequestMapping(value = "/enable/{id}", method = {RequestMethod.GET})
    public ResultBean<Boolean> enableById(@PathVariable Long id) {
        return new ResultBean<Boolean>(expressDeptService.enableById(id));
    }

    /**
     * 通过id删除单条数据
     *
     * @param id
     * @return
     */
    @RequestMapping(value = "/delete/account/{id}", method = {RequestMethod.GET})
    public ResultBean<Boolean> deleteAccountById(@PathVariable Long id) {
        return new ResultBean<Boolean>(expressDeptService.deleteAccountById(id));
    }

    /**
     * 通过id启用
     *
     * @param id
     * @return
     */
    @RequestMapping(value = "/enable/account/{id}", method = {RequestMethod.GET})
    public ResultBean<Boolean> enableAccountById(@PathVariable Long id) {
        return new ResultBean<Boolean>(expressDeptService.enableAccountById(id));
    }

    @PostMapping("/getConsumeRecords")
    public ResultBean<Pager<ExpressAccountConsumeRecordDto>> getConsumeRecords(@RequestBody String string) {
        return new ResultBean<>(expressDeptService.getConsumeRecords(JSON.parseObject(JSON.parseObject(string, RequestBean.class).getData(), ExpressDeptFilter.class)));
    }

    /**
     * @description 快递管理-快件状态查询
     * @author sunjj
     * @updateTime
     * @throws
     */
    @PostMapping("/getDeliveries")
    public ResultBean<Pager<ExpressAccountConsumeRecordDto>> getDeliveries(@RequestBody String string){
        return new ResultBean<>(expressDeptService.getDeliveries(JSON.parseObject(JSON.parseObject(string, RequestBean.class).getData(), ExpressDeptFilter.class)));
    }


}
