package cn.bs.bsmanagebasic.controller;

import cn.bs.bsmanagebasic.entity.oss.BucketFilter;
import cn.bs.bsmanagebasic.entity.oss.OssFileInfoDto;
import cn.bs.bsmanagebasic.service.FileManagementService;
import cn.bs.bsmanagecommon.requst.RequestBean;
import cn.bs.bsmanagecommon.result.ResultBean;
import com.alibaba.fastjson.JSON;
import com.aliyun.oss.model.BucketInfo;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

/**
 * @packageName: cn.bs.bsmanagebasic.controller
 * @className: FileManagementController
 * @description: 文件管理功能(OSS)
 * @author: liujs
 * @date: 2019/12/16 0016 下午 13:51
 * @version: 1.0
 */
@RestController
@RequestMapping("/fileManagement")
@CrossOrigin
public class FileManagementController {
    @Resource
    private FileManagementService fileManagementService;

    /**
     * 创建存储空间
     * @param data
     * @return
     */
    @RequestMapping(value = "/bucket/add",method = RequestMethod.POST)
    public ResultBean<String> addBucket(@RequestBody String data){
        return new ResultBean<>(fileManagementService.addBucket(JSON.parseObject(JSON.parseObject(data, RequestBean.class).getData(), BucketFilter.class)));
    }

    /**
     * 删除存储空间
     * @param data
     * @return
     */
    @RequestMapping(value = "/bucket/delete",method = RequestMethod.POST)
    public ResultBean<Boolean> deleteBucket(@RequestBody String data){
        return new ResultBean<>(fileManagementService.deleteBucket(JSON.parseObject(JSON.parseObject(data, RequestBean.class).getData(), BucketFilter.class)));
    }

    /**
     * 获取存储空间名称
     * @param data
     * @return
     */
    @RequestMapping(value = "/bucket/list",method = RequestMethod.POST)
    public ResultBean<List<String>> bucketList(@RequestBody String data){
        return new ResultBean<>(fileManagementService.bucketList(JSON.parseObject(JSON.parseObject(data, RequestBean.class).getData(), BucketFilter.class)));
    }

    /**
     * 获取存储空间的信息
     * @param data
     * @return
     */
    @RequestMapping(value = "/bucket/info",method = RequestMethod.POST)
    public ResultBean<BucketInfo> bucketInfo(@RequestBody String data){
        return new ResultBean<>(fileManagementService.bucketInfo(JSON.parseObject(JSON.parseObject(data, RequestBean.class).getData(), BucketFilter.class)));
    }

    /**
     * 存储空间是否存在
     * @param data
     * @return
     */
    @RequestMapping(value = "/bucket/isExist",method = RequestMethod.POST)
    public ResultBean<Boolean> isExistBucket(@RequestBody String data){
        return new ResultBean<>(fileManagementService.isExistBucket(JSON.parseObject(JSON.parseObject(data, RequestBean.class).getData(), BucketFilter.class)));
    }

    /**
     * 修改存储空间的权限
     * @param data
     * @return
     */
    @RequestMapping(value = "/bucket/update",method = RequestMethod.POST)
    public ResultBean<Boolean> updateBucket(@RequestBody String data){
        return new ResultBean<>(fileManagementService.updateBucket(JSON.parseObject(JSON.parseObject(data, RequestBean.class).getData(), BucketFilter.class)));
    }

    /**
     * 文件上传(OSS简单上传_文件上传)
     * @param data
     * @return
     */
    @RequestMapping(value = "file/upload",method = RequestMethod.POST)
    public ResultBean<Boolean> uploadFile(@RequestBody String data){
        return new ResultBean<>(fileManagementService.uploadFile(JSON.parseObject(JSON.parseObject(data, RequestBean.class).getData(), BucketFilter.class)));
    }

    /**
     * 文件下载(OSS下载到本地文件)
     * @param data
     * @return
     */
    @RequestMapping(value = "file/download",method = RequestMethod.POST)
    public ResultBean<Boolean> downloadFile(@RequestBody String data){
        return new ResultBean<>(fileManagementService.downloadFile(JSON.parseObject(JSON.parseObject(data, RequestBean.class).getData(), BucketFilter.class)));
    }

    /**
     * 删除文件(批量删除)
     * @param data
     * @return
     */
    @RequestMapping(value = "/file/delete",method = RequestMethod.POST)
    public ResultBean<List<String>> deleteFile(@RequestBody String data){
        return  new ResultBean<>(fileManagementService.deleteFile(JSON.parseObject(JSON.parseObject(data,RequestBean.class).getData(),BucketFilter.class)));
    }

    /**
     * 获取文件详情
     * @param data
     * @return
     */
    @RequestMapping(value = "file/info",method = RequestMethod.POST)
    public ResultBean<OssFileInfoDto> fileInfo(@RequestBody String data){
        return new ResultBean<>(fileManagementService.fileInfo(JSON.parseObject(JSON.parseObject(data,RequestBean.class).getData(),BucketFilter.class)));
    }

    /**
     * 修改文件读写权限 存储权限
     * @param data
     * @return
     */
    @RequestMapping(value = "file/update",method = RequestMethod.POST)
    public ResultBean<Boolean> updateFile(@RequestBody String data){
        return new ResultBean<>(fileManagementService.updateFile(JSON.parseObject(JSON.parseObject(data,RequestBean.class).getData(),BucketFilter.class)));
    }

    /**
     * 获取所有文件名称
     * @param data
     * @return
     */
    @RequestMapping(value = "file/list",method = RequestMethod.POST)
    public ResultBean<List<OssFileInfoDto>> fileList(@RequestBody String data){
        return new ResultBean<>(fileManagementService.fileList(JSON.parseObject(JSON.parseObject(data,RequestBean.class).getData(),BucketFilter.class)));
    }
}
