package cn.bs.bsmanagebasic.controller;

import cn.bs.bsmanagebasic.common.enums.AdminTypeEnum;
import cn.bs.bsmanagebasic.common.util.Pager;
import cn.bs.bsmanagebasic.entity.User;
import cn.bs.bsmanagebasic.entity.dto.RoleListDto;
import cn.bs.bsmanagebasic.entity.dto.RoleMenuBindDto;
import cn.bs.bsmanagebasic.entity.sys.Role;
import cn.bs.bsmanagebasic.service.RoleService;
import cn.bs.bsmanagecommon.result.ResultBean;
import com.alibaba.fastjson.JSON;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.util.Date;
import java.util.List;

/**
 * @author LiuJS
 * @Time 2021/4/26 17:45
 * @Desc
 * @Version 1.0
 */
@RestController
@RequestMapping("role")
public class RoleController {

    @Autowired
    private RoleService roleService;

    @GetMapping("list")
    public ResultBean<Pager<Role>> ListRoles(HttpServletRequest request, RoleListDto roleListDto) {
        HttpSession session = request.getSession();
        User user = JSON.parseObject(session.getAttribute(session.getId()).toString(), User.class);
        roleListDto.setParentId(user.getRoleId());
        roleListDto.setRoleType(user.getAdminType());
        roleListDto.setStoreId(user.getStoreId());
        return new ResultBean(roleService.listRole(request, roleListDto));
    }
    @GetMapping("all")
    public ResultBean<List<Role>> ListRoles(HttpServletRequest request) {
        RoleListDto roleListDto = new RoleListDto();

        HttpSession session = request.getSession();
        User user = JSON.parseObject(session.getAttribute(session.getId()).toString(), User.class);
        roleListDto.setParentId(user.getRoleId());
        roleListDto.setRoleType(user.getAdminType());
        roleListDto.setStoreId(user.getStoreId());
        return new ResultBean(roleService.listRoleAll(roleListDto));
    }

    @GetMapping("get")
    public ResultBean getRole(Long id) {
        return new ResultBean(roleService.getRole(id));
    }

    @PostMapping("add")
    public ResultBean addRole(HttpServletRequest request,@RequestBody Role role) {
        HttpSession session = request.getSession();
        User user = JSON.parseObject(session.getAttribute(session.getId()).toString(), User.class);
        role.setRoleType(user.getAdminType());
        role.setParentId(user.getRoleId());
        role.setStoreId(user.getStoreId());
        return new ResultBean(roleService.addRole(request, role));
    }

    @PostMapping("update")
    public ResultBean updateRole(HttpServletRequest request, @RequestBody Role role) {
        HttpSession session = request.getSession();
        User user = JSON.parseObject(session.getAttribute(session.getId()).toString(), User.class);
        role.setUpdateTime(new Date());
        role.setRoleType(user.getAdminType());
        role.setStoreId(user.getStoreId());
        return new ResultBean(roleService.updateRole(request, role));
    }

    @GetMapping("delete")
    public ResultBean deleteRole(HttpServletRequest request, Long id) {
        return roleService.deleteRole(request, id);
    }

    @GetMapping("get_menu")
    public ResultBean getRoleMenu(HttpServletRequest request,Long roleId) {
        return new ResultBean(roleService.getRoleMenu(request, roleId));
    }

    /**
     * 角色绑定菜单
     * @param roleMenuBindDto
     * @return
     */
    @PostMapping("bind_menu")
    public ResultBean bindRoleMenu(@RequestBody RoleMenuBindDto roleMenuBindDto) {
        return roleService.bindRoleMenu(roleMenuBindDto);
    }

    /**
     * 查询角色拥有的模版权限菜单
     * @param roleId
     * @return
     */
    @GetMapping("get_role_template_menu")
    public ResultBean getRoleTemplateMenu(Long roleId) {
        return new ResultBean(roleService.getRoleTemplateMenu(roleId));
    }

    /**
     * 查询门店角色模版列表
     * @param request
     * @param roleDto
     * @return
     */
    @GetMapping("storeRoleTemplate/list")
    public ResultBean<Pager<Role>> listStoreRoleTemplate(HttpServletRequest request, RoleListDto roleDto) {
        HttpSession session = request.getSession();
        User user = JSON.parseObject(session.getAttribute(session.getId()).toString(), User.class);
        roleDto.setRoleType(AdminTypeEnum.STORE_ADMIN_TEMPLATE.getType());
        roleDto.setStoreId(user.getStoreId());
        return new ResultBean(roleService.listRole(request,roleDto));
    }
    /**
     * 门店角色模版 添加
     * @param request
     * @param role
     * @return
     */
    @PostMapping("storeRoleTemplate/add")
    public ResultBean addStoreRoleTemplate(HttpServletRequest request,@RequestBody Role role) {
        HttpSession session = request.getSession();
        User user = JSON.parseObject(session.getAttribute(session.getId()).toString(), User.class);
        role.setRoleType(AdminTypeEnum.STORE_ADMIN_TEMPLATE.getType());
        role.setStoreId(user.getStoreId());
        role.setParentId(0L);
        return new ResultBean(roleService.addRole(request, role));
    }
    /**
     * 门店角色模版 修改
     * @param request
     * @param role
     * @return
     */
    @PostMapping("storeRoleTemplate/update")
    public ResultBean<Pager<Role>> updateStoreRoleTemplate(HttpServletRequest request, @RequestBody Role role) {
        HttpSession session = request.getSession();
        User user = JSON.parseObject(session.getAttribute(session.getId()).toString(), User.class);
        role.setRoleType(AdminTypeEnum.STORE_ADMIN_TEMPLATE.getType());
        role.setStoreId(user.getStoreId());
        return new ResultBean(roleService.updateRole(request, role));
    }
}
