package cn.bs.bsmanagebasic.controller;

import cn.bs.bsmanagebasic.common.util.Pager;
import cn.bs.bsmanagebasic.entity.User;
import cn.bs.bsmanagebasic.entity.dto.StoreStaffDto;
import cn.bs.bsmanagebasic.entity.filter.StoreStaffFilter;
import cn.bs.bsmanagebasic.service.StoreStaffService;
import cn.bs.bsmanagecommon.result.ResultBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 * 门店外派员
 */

@RestController
@RequestMapping("/storeStaff")
@CrossOrigin(allowCredentials ="true")
public class StoreStaffController {

    @Autowired
   private StoreStaffService storeStaffService;

    /**
     * 获取外派员信息
     * @param
     * @return
     */
    @GetMapping(value ="/list")
    public ResultBean<List<StoreStaffDto>> getStoreStaffList(){
        return new ResultBean(storeStaffService.getStoreStaffList());
    }

    @GetMapping(value ="/page")
    public ResultBean<Pager<StoreStaffDto>> getStoreStaffPage(StoreStaffFilter filter){
        return new ResultBean(storeStaffService.getStoreStaffPage(filter));
    }

    /**
     * 新增外派员
     * @param user
     * @return
     */
    @RequestMapping(value ="/addStaff",method = {RequestMethod.POST})
    public ResultBean addStoreStaff(@RequestBody User user){
        return new ResultBean(storeStaffService.addStoreStaff(user));
    }

    /**
     * 修改外派员信息
     * @param storeStaff
     * @return
     */
    @RequestMapping(value ="/updateStaff",method = {RequestMethod.POST})
    public ResultBean updateStoreStaffById(@RequestBody User storeStaff){
        return new ResultBean(storeStaffService.updateStoreStaffById(storeStaff));
    }

    /**
     * 删除
     * @param userId
     * @return
     */
    @RequestMapping(value ="/deleteStaff",method = {RequestMethod.POST})
    public ResultBean deleteStoreStaffById(String userId){
        return new ResultBean(storeStaffService.deleteStoreStaffById(userId));
    }


    /**
     * 查询
     * @param storeStaff
     * @return
     */
    @RequestMapping(value ="/queryStaff",method = {RequestMethod.GET})
    public ResultBean<List<StoreStaffDto>> queryStoreStaff(StoreStaffFilter storeStaff){
        return new ResultBean<>(storeStaffService.queryStoreStaff(storeStaff));
    }

    /**
     * 详情
     * @param userId
     * @return
     */
    @RequestMapping(value ="/detailStaff",method = {RequestMethod.GET})
    public ResultBean<StoreStaffDto> detailStaff(String userId){
        return new ResultBean(storeStaffService.detailStaff(userId));
    }
}
