package cn.bs.bsmanagebasic.controller;

import cn.bs.bsmanagebasic.entity.sys.SysParam;
import cn.bs.bsmanagebasic.entity.sys.SysType;
import cn.bs.bsmanagebasic.service.SystemService;
import cn.bs.bsmanagecommon.requst.RequestBean;
import cn.bs.bsmanagecommon.result.ResultBean;
import com.alibaba.fastjson.JSON;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 〈功能概述〉<br>
 *
 * @className: SysParamController
 * @package: cn.bs.bsmanagebasic.controller
 * @author: kael
 * @date: 2019/12/2 0002 上午 10:54
 */
@RestController
@RequestMapping("/system")
@CrossOrigin(allowCredentials ="true")
public class SysParamController {

    @Autowired
    private SystemService systemService;

    /**
     * 获取快递和平台
     * @return
     */
    @GetMapping("/getExpressAndPlatform")
    public ResultBean<List<SysParam>> getExpressAndPlatform(){
        return new ResultBean<List<SysParam>>(systemService.getExpressAndPlatform());
    }

    /**
     * 获取系统参数
     * @return
     */
    @GetMapping("/getAllListType")
    public ResultBean<List<SysType>> getAllListType(){
        return new ResultBean<List<SysType>>(systemService.getAllListType());
    }

    /**
     * 通过type获取参数
     *
     * @param id
     * @return
     */
    @RequestMapping(value = "/getDataByType/{type}", method = {RequestMethod.GET})
    public ResultBean<List<SysParam>> getDataByType(@PathVariable Integer type) {
        return new ResultBean<List<SysParam>>(systemService.getDataByType(type));
    }

    /**
     * 通过id启用禁用
     *
     * @param id
     * @return
     */
    @RequestMapping(value = "/type/enable/{id}", method = {RequestMethod.GET})
    public ResultBean<Boolean> typeEnable(@PathVariable Integer id) {
        return new ResultBean<Boolean>(systemService.typeEnable(id));
    }

    /**
     * 保存
     *
     * @param id
     * @return
     */
    @RequestMapping(value = "/type/save", method = {RequestMethod.POST})
    public ResultBean<Boolean> saveType(@RequestBody String data) {
        return new ResultBean<Boolean>(systemService.saveType(JSON.parseObject(JSON.parseObject(data, RequestBean.class).getData(), SysType.class)));
    }
    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @RequestMapping(value = "/type/delete/{id}", method = {RequestMethod.GET})
    public ResultBean<Boolean> deleteType(@PathVariable Integer id) {
        return new ResultBean<Boolean>(systemService.deleteType(id));
    }

    /**
     * 保存参数
     *
     * @param id
     * @return
     */
    @RequestMapping(value = "/param/save", method = {RequestMethod.POST})
    public ResultBean<Boolean> saveParam(@RequestBody String data) {
        return new ResultBean<Boolean>(systemService.saveParam(JSON.parseObject(JSON.parseObject(data, RequestBean.class).getData(), SysParam.class)));
    }

    /**
     * 通过id删除参数
     *
     * @param id
     * @return
     */
    @RequestMapping(value = "/param/delete/{id}", method = {RequestMethod.GET})
    public ResultBean<Boolean> deleteParam(@PathVariable Integer id) {
        return new ResultBean<Boolean>(systemService.deleteParam(id));
    }

    /**
     * 通过id启用禁用
     *
     * @param id
     * @return
     */
    @RequestMapping(value = "/param/enable/{id}/{typeId}", method = {RequestMethod.GET})
    public ResultBean<Boolean> enableParam(@PathVariable Integer id,@PathVariable Integer typeId) {
        return new ResultBean<Boolean>(systemService.enableParam(id,typeId));
    }

}
