package cn.bs.bsmanagebasic.listener;

import cn.bs.bsmanagebasic.entity.ApiLoggingEvent;
import cn.bs.bsmanagebasic.service.ApiLogService;
import com.alibaba.fastjson.JSONObject;
import com.rabbitmq.tools.json.JSONUtil;


import lombok.extern.slf4j.Slf4j;
import com.rabbitmq.client.Channel;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;


/**
 * @ProjectName: bs-manage
 * @Package: cn.bs.bsmanagebasic.listener
 * @ClassName: RabbitMQLogListener
 * @Author: kexl
 * @Description: ${description}
 * @Date: 2019/10/22 10:39
 * @Version: 1.0
 */
@Slf4j
@Component
public class RabbitMQLogListener {

    @Autowired
    private ApiLogService apiLogService;

    @RabbitListener(bindings = @QueueBinding(
            value = @Queue(value = "${spring.rabbitmq.ApiLogQueueName}",durable = "true"),
            exchange = @Exchange(name="${spring.rabbitmq.ApiLogExchangeName}",durable = "true",type = "direct"),
            key = "${spring.rabbitmq.ApiLogRoutingKey}"
    ))

    @RabbitHandler
    public void directHandlerManualAck(String json, Message message, Channel channel) {
        //  如果手动ACK,消息会被监听消费,但是消息在队列中依旧存在,如果 未配置 acknowledge-mode 默认是会在消费完毕后自动ACK掉
        final long deliveryTag = message.getMessageProperties().getDeliveryTag();
        try {
            log.info("直接队列，手动ACK，接收消息：{}", JSONObject.parseObject(json, ApiLoggingEvent.class));
            ApiLoggingEvent apiLog = JSONObject.parseObject(json, ApiLoggingEvent.class);

            apiLogService.addApiLog(apiLog);
            // 通知 MQ 消息已被成功消费,可以ACK了
            channel.basicAck(deliveryTag, false);
        } catch (IOException e) {
            try {
                // 处理失败,重新压入MQ
                channel.basicRecover();
            } catch (IOException e1) {
                e1.printStackTrace();
            }
        }
    }
}
