package cn.bs.bsmanagebasic.service;


import cn.bs.bsmanagebasic.entity.delivery.Delivery;
import cn.bs.bsmanagebasic.entity.dto.EsDeliveryDto;
import cn.bs.bsmanagebasic.entity.filter.DeliveryFilter;
import lombok.Data;

import java.io.Serializable;
import java.util.List;

/**
 * @author sunjj
 * @version 1.0.0
 * @ClassName DeliverySyncESService.java
 * @Description Delivery ES服务相关（数据同步，查询等），
 * 在‘com.parksong.basics.aspect.EnableESAop’进行切面处理，
 * 根据配置（es-server.enable）判断是否启用ES服务，
 * 如果不启用ES服务,则所有方法不会被执行
 * @createTime 2019年10月21日 11:01:00
 */
public interface DeliveryESService {

    /**
     * 插入一条数据到ES中
     *
     * @param delivery
     */
    void insert(Delivery delivery);

    /**
     * 更新ES中的数据
     *
     * @param dto
     */
    void update(EsDeliveryDto dto);

    /**
     * 删除ES中的数据
     *
     * @param dto
     */
//    void delete(EsDeliveryDto dto);

    /**
     * 根据delivery更新ES中的信息
     *
     * @param delivery
     */

//    void updateByDelivery(Delivery delivery);

    /**
     * 根据DeliveryFilter从ES中查询Delivery信息
     *
     * @param filter
     * @return
     */
    ResponsePage selectByFilter(DeliveryFilter filter);

    Long countByFilter(DeliveryFilter filter);

    @Data
    class ResponseBean implements Serializable {
        private String msg;
        private int code;
        private ResponsePage data;
        private String requestSign;
    }

    @Data
    class ResponsePage {
        private Long total;
        private int pageSize;
        private int pages;
        private int pageNumber;
        private List<Delivery> list;
    }
}
