package cn.bs.bsmanagebasic.service.impl;

import cn.bs.bsmanagebasic.common.util.Pager;
import cn.bs.bsmanagebasic.dao.jisu.AdminUserMapper;
import cn.bs.bsmanagebasic.entity.User;
import cn.bs.bsmanagebasic.entity.dto.AdminUserEditDto;
import cn.bs.bsmanagebasic.entity.dto.AdminUserListDto;
import cn.bs.bsmanagebasic.entity.user.AdminUser;
import cn.bs.bsmanagebasic.entity.user.UserRole;
import cn.bs.bsmanagebasic.entity.vo.AdminUserVo;
import cn.bs.bsmanagebasic.service.AdminUserService;
import cn.bs.bsmanagebasic.utils.Md5Util;
import cn.bs.bsmanagecommon.exception.BusinessException;
import com.alibaba.fastjson.JSON;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.util.Date;

/**
 * @description:
 * @author: chenxuebing
 * @create: 2021-05-07 14:58
 **/
@Service
public class AdminUserServiceImpl implements AdminUserService {

    @Resource
    private AdminUserMapper adminUserMapper;

    @Override
    public Pager<AdminUser> listAdminUser(HttpServletRequest request, AdminUserListDto adminUserListDto) {
        adminUserListDto.initSort();
        adminUserListDto.initPage();
        HttpSession session = request.getSession();
        User user = JSON.parseObject(session.getAttribute(session.getId()).toString(), User.class);
        adminUserListDto.setStoreId(user.getStoreId());
        int count = adminUserMapper.getAdminUserCount(adminUserListDto);
        Pager<AdminUser> pager = new Pager(count, adminUserListDto.getPageNumber(), adminUserListDto.getPageSize());
        if(count > 0) {
            pager.setList(adminUserMapper.listAdminUser(adminUserListDto));
        }
        return pager;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean addAdminUser(HttpServletRequest request, AdminUserEditDto adminUserEditDto) {
        if (adminUserEditDto.getPassword() == null || adminUserEditDto.getPassword().equals("")) {
            throw new BusinessException("密码不能为空");
        }
        adminUserEditDto.setPassword(Md5Util.md5AsLowerHex(adminUserEditDto.getPassword()));
        HttpSession session = request.getSession();
        User user = JSON.parseObject(session.getAttribute(session.getId()).toString(), User.class);
        adminUserEditDto.setStoreId(user.getStoreId());
        adminUserEditDto.setAdminType(adminUserEditDto.getRoleId() == 4 ? 3 : user.getAdminType());
        adminUserEditDto.setCreateTime(new Date());
        adminUserMapper.addAdminUser(adminUserEditDto);
        UserRole userRole = new UserRole();
        userRole.setRoleId(adminUserEditDto.getRoleId());
        userRole.setUserId(adminUserEditDto.getUserId());
        userRole.setCreateTime(new Date());
        userRole.setStoreId(user.getStoreId());
        userRole.setAdminType(adminUserEditDto.getRoleId() == 4 ? 3 : user.getAdminType());
        adminUserMapper.addAdminUserRole(userRole);

        return true;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateAdminUser(HttpServletRequest request, AdminUserEditDto adminUserEditDto) {
        HttpSession session = request.getSession();
        User user = JSON.parseObject(session.getAttribute(session.getId()).toString(), User.class);
        adminUserEditDto.setStoreId(user.getStoreId());
        if (adminUserEditDto.getPassword() != null && !adminUserEditDto.getPassword().equals("")) {
            adminUserEditDto.setPassword(Md5Util.md5AsLowerHex(adminUserEditDto.getPassword()));
        }
        adminUserMapper.updateAdminUser(adminUserEditDto);
        adminUserMapper.deleteUserRole(adminUserEditDto);
        UserRole userRole = new UserRole();
        userRole.setRoleId(adminUserEditDto.getRoleId());
        userRole.setUserId(adminUserEditDto.getUserId());
        userRole.setCreateTime(new Date());
        userRole.setStoreId(user.getStoreId());
        userRole.setAdminType(adminUserEditDto.getRoleId() == 4 ? 3 : user.getAdminType());
        adminUserMapper.addAdminUserRole(userRole);
        return true;
    }

    @Override
    public AdminUserVo getAdminUser(HttpServletRequest request, String userId) {
        AdminUser adminUser = new AdminUser();
        HttpSession session = request.getSession();
        User user = JSON.parseObject(session.getAttribute(session.getId()).toString(), User.class);
        adminUser.setUserId(userId);
        adminUser.setStoreId(user.getStoreId());
        return adminUserMapper.getAdminUser(adminUser);
    }

    @Override
    public int deleteAdminUser(HttpServletRequest request, String userId) {
        AdminUser adminUser = new AdminUser();
        HttpSession session = request.getSession();
        User user = JSON.parseObject(session.getAttribute(session.getId()).toString(), User.class);
        adminUser.setUserId(userId);
        adminUser.setStoreId(user.getStoreId());
        return adminUserMapper.deleteAdminUser(adminUser);
    }

}
