package cn.bs.bsmanagebasic.service.impl;

import cn.bs.bsmanagebasic.common.constants.DeliveryStatusConstants;
import cn.bs.bsmanagebasic.common.enums.DeliverySignReturnEnum;
import cn.bs.bsmanagebasic.common.enums.DeliveryStatusEnum;
import cn.bs.bsmanagebasic.common.enums.SMSErrorCodeEnum;
import cn.bs.bsmanagebasic.common.util.Pager;
import cn.bs.bsmanagebasic.config.RabbitConfig;
import cn.bs.bsmanagebasic.dao.jeesite.ExStoreMapper;
import cn.bs.bsmanagebasic.dao.jisu.DeliveryMapper;
import cn.bs.bsmanagebasic.entity.*;
import cn.bs.bsmanagebasic.entity.bases.BatchDataMsg;
import cn.bs.bsmanagebasic.entity.delivery.Delivery;
import cn.bs.bsmanagebasic.entity.dto.EsDeliveryDto;
import cn.bs.bsmanagebasic.entity.dto.StoreDto;
import cn.bs.bsmanagebasic.entity.filter.DeliveryFilter;
import cn.bs.bsmanagebasic.entity.store.SignReturnBean;
import cn.bs.bsmanagebasic.service.*;
import cn.bs.bsmanagecommon.exception.BusinessException;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 〈功能概述〉<br>
 *
 * @className: DeliveryServiceImpl
 * @package: cn.bs.bsmanagebasic.service
 * @author: kael
 * @date: 2019/11/6 0006 下午 15:43
 */
@Service
@Slf4j
public class DeliveryServiceImpl implements DeliveryService {

    @Autowired
    private DeliveryMapper deliveryMapper;

    @Autowired
    private ExStoreMapper exStoreMapper;

    @Autowired
    private RabbitService rabbitService;

    @Autowired
    private RabbitConfig rabbitConfig;

    @Autowired
    private DeliveryESService deliveryESService;

    @Autowired
    private StoreManageService storeManageService;

    @Autowired
    private DeliveryRunningWaterService deliveryRunningWaterService;

    @Autowired
    private ExpressDeptAccountConsumeService expressConsumeService;

    @Autowired
    private StoreShelfService storeShelfService;

    @Autowired
    private ExpressService expressService;

    @Override
    public List<String> getCount(PickupQueue pickupQueue) {
        return deliveryMapper.getCount(pickupQueue);
    }

    @Override
    public List<BatchDataMsg> sign(DeliveryFilter filter) {
        Map<Integer, List<Delivery>> deliveryMap = new HashMap<>(100);
        List<BatchDataMsg> batchDataMsgs = new ArrayList<>();
        if (filter.getStoreId() == null) {
            List<ExStore> stores = exStoreMapper.selectByExample(new ExStoreExample());
            DeliveryFilter filterStore = null;
            stores.forEach(store -> {
                if (store.getId().compareTo(1001L) == 0) {
                    return;
                }
                filter.setStoreId(store.getId().intValue());
                List<Delivery> deliveries = null;
                try {
                    deliveries = deliveryMapper.getListByFilter(filter);
                } catch (Exception e) {
                    log.info("没有编号为：{}的门店", store.getId().toString());
                    return;
                }
                if (deliveries == null || deliveries.size() == 0) {
                    return;
                }
                deliveryMap.put(filter.getStoreId(), deliveries);
            });
        } else if (filter.getStoreIds() != null && filter.getStoreIds().size() > 0) {
            filter.getStoreIds().forEach(storeId -> {
                filter.setStoreId(storeId);
                putDeliveryMap(filter, deliveryMap);
            });
        } else {
            putDeliveryMap(filter, deliveryMap);
        }
        deliveryMap.forEach((key, value) -> {
            StringBuffer stringBuffer = new StringBuffer();
            value.forEach(delivery -> {
                if (key.compareTo(1095) == 0) {
                    System.out.println(key);
                }
                try {
                    String status = "";
                    if (delivery.getReturnSignStatus().compareTo(2) == 0) {
                        status = "out";
                    } else if (delivery.getReturnSignStatus().compareTo(4) == 0 || delivery.getReturnSignStatus().compareTo(0) == 0) {
                        status = "in";
                    }
                    if ("".equals(status)) {
                        stringBuffer.append(delivery.getId()).append(",");
                        return;
                    }
                    this.signBack(delivery.getExpId(), delivery.getExpcode(), delivery.getUserPhone(), delivery.getStoreId(), status, (delivery.getShelfId() == null ? "" : delivery.getShelfId() + ""), delivery.getShelfCode(), delivery.getPlatform());
                } catch (Exception e) {
                    e.printStackTrace();
                    stringBuffer.append(delivery.getId()).append(",");
                }
            });
            if (!Strings.isBlank(stringBuffer.toString())) {
                batchDataMsgs.add(BatchDataMsg.builder()
                        .id(key.toString())
                        .result(false)
                        .msg(stringBuffer.toString())
                        .build());
            }
        });
        return batchDataMsgs;
    }

    private void putDeliveryMap(DeliveryFilter filter, Map<Integer, List<Delivery>> deliveryMap) {
        List<Delivery> deliveries = deliveryMapper.getListByFilter(filter);
        if (deliveries == null || deliveries.size() == 0) {
            return;
        }
        deliveryMap.put(filter.getStoreId(), deliveries);
    }

    @Override
    public void signBack(Integer expId, String expCode, String userPhone, Integer storeId, String status, String shelfId, String shelfCode, String platform) {
        rabbitService.send(rabbitConfig.getExpressExchange(), rabbitConfig.getExpressKey(), JSON.toJSONString(SignReturnBean.builder()
                .expId(expId)
                .expCode(expCode)
                .telPhone(userPhone)
                .store(exStoreMapper.selectByPrimaryKey(storeId.longValue()))
                .actionType(status)
                .shelfId(shelfId)
                .shelfCode(shelfCode)
                .expressSignBean(exStoreMapper.getSignBackSwitch(expId, storeId))
                //判断是否开启调用居欢颜
                .jhySignBean(exStoreMapper.getSignBackSwitch(9999, storeId))
                .platform(platform)
                .build()));
    }

    @Override
    public Pager<Delivery> queryDeliverys(DeliveryFilter deliveryFilter, HttpServletRequest request) {
        HttpSession session = request.getSession();
        User user = JSON.parseObject(session.getAttribute(session.getId()).toString(), User.class);
        deliveryFilter.setStoreId(user.getStoreId().intValue());
        deliveryFilter.initPage();
        // 设置默认查询条件
        if (null == deliveryFilter.getStartTime()) {
            int timePeriod = -3;
            LocalDateTime now = LocalDateTime.now();
            LocalDateTime startTime =
                    LocalDateTime.of(now.getYear(), now.getMonth(), now.getDayOfMonth(), 0, 0)
                            .plusMonths(timePeriod);
            deliveryFilter.setStartTime(startTime);
        }
        if (null == deliveryFilter.getTimeType()) {
            deliveryFilter.setTimeType(1);
        }

        List<Integer> storeIds = null;
        boolean matchAddress = false;
        if (!Strings.isBlank(deliveryFilter.getArea())) {
            storeIds = storeManageService.selectIdByRegionCode(deliveryFilter.getArea());
            matchAddress = true;
        } else if (!Strings.isBlank(deliveryFilter.getCity())) {
            storeIds = storeManageService.selectIdByRegionCode(deliveryFilter.getCity());
            matchAddress = true;
        } else if (!Strings.isBlank(deliveryFilter.getProvince())) {
            storeIds = storeManageService.selectIdByRegionCode(deliveryFilter.getProvince());
            matchAddress = true;
        }
        if (matchAddress && (null == storeIds || storeIds.size() == 0)) {
            return new Pager<>(0, 1, deliveryFilter.getPageSize());
        }
        if (null != deliveryFilter.getStoreId()) {
            if(0 == deliveryFilter.getStoreId()){
                storeIds = null;
            }else {
                if (null == storeIds) {
                    storeIds = new ArrayList<>();
                }
                storeIds.add(deliveryFilter.getStoreId());
            }
        }

        if (null != storeIds) {
            deliveryFilter.setStoreIds(storeIds);
        }

        Pager<Delivery> pager = null;
        Long total = deliveryMapper.countByFilterFromMaster(deliveryFilter);
        if (null == total || total.compareTo(0L) == 0) {
            pager = new Pager<>(0, 1, deliveryFilter.getPageSize());
        } else {
            List<Delivery> deliveries = deliveryMapper.selectByFilterFromMaster(deliveryFilter);
            pager = new Pager<>(total, deliveryFilter.getPageNumber(), deliveryFilter.getPageSize());
            pager.setList(returnDeliveryHandle(deliveries));
        }
        return pager;
    }

    @Override
    public List<BatchDataMsg> updateExpStatus(DeliveryFilter filter) {
        List<BatchDataMsg> batchDataMsgs = new ArrayList<>();
        List<EsDeliveryDto> esDeliveryDtoList = new ArrayList<>();
        List<UpdateDeliveryInsertCondition> updateDeliveryInsertConditionList = new ArrayList<>();
        if (null == filter.getIdAndStoreIds() || filter.getIdAndStoreIds().size() < 1 || null == filter.getUpdateStatus()) {
            throw new BusinessException("参数错误");
        }
        try {
            filter.getIdAndStoreIds().forEach(idAndStoreIds -> {
                EsDeliveryDto esDeliveryDto = new EsDeliveryDto();
                Delivery oldDelivery = new Delivery();
                Delivery newDelivery = new Delivery();
                //TODO 暂时只有管理员用户操作这个功能 useId=13812345678 后期再修改
                String useId = "13812345678";
                //获取修改前的delivery
                BeanUtils.copyProperties(idAndStoreIds, oldDelivery);
                //修改的字段 为空不修改,不为空就修改
                idAndStoreIds.setReason(filter.getReason().length() < 41 ? filter.getReason() : filter.getReason().substring(0, 40));
                idAndStoreIds.setStatus(filter.getUpdateStatus());
                //获取修改ES的esDeliveryDto
                BeanUtils.copyProperties(idAndStoreIds, esDeliveryDto);
                esDeliveryDtoList.add(esDeliveryDto);
                //获取修改后的delivery
                BeanUtils.copyProperties(idAndStoreIds, newDelivery);
                updateDeliveryInsertConditionList.add(UpdateDeliveryInsertCondition.builder()
                        .oldDelivery(oldDelivery)
                        .newDelivery(newDelivery)
                        .useId(useId)
                        .appVersion("")
                        .apiVersion("")
                        .build());
            });
            //分别修改MySQL数据库中delivery表和对应门店业务表中信息
            deliveryMapper.updateExpStatus(filter.getIdAndStoreIds());
            deliveryMapper.updateExpStatusFromDelivery(filter.getIdAndStoreIds());
            //修改ES数据库中快递信息
            esDeliveryDtoList.forEach(esDeliveryDto -> deliveryESService.update(esDeliveryDto));
            //添加订单流水
            updateDeliveryInsertConditionList.forEach(updateDeliveryInsertCondition ->
                    //TODO 暂时修改快递状态是管理员账号操作 storeId=110
                    deliveryRunningWaterService.updateDeliveryInsertByAdmin(updateDeliveryInsertCondition.getOldDelivery(), updateDeliveryInsertCondition.getNewDelivery(), 110, updateDeliveryInsertCondition.getUseId(), updateDeliveryInsertCondition.getAppVersion(), updateDeliveryInsertCondition.getApiVersion()));
            expressConsumeService.updateExpStatus(filter);
        } catch (Exception e) {
            e.printStackTrace();
            throw new BusinessException("出现异常");
        }
        batchDataMsgs.add(BatchDataMsg.builder()
                .result(true)
                .msg("修改成功")
                .build());
        return batchDataMsgs;
    }

    @Override
    public Integer queryIdFromMasterDelivery(Delivery delivery) {
        return deliveryMapper.queryIdFromMasterDelivery(delivery);
    }

    @Override
    public Delivery getByMasterId(Integer id) {
        return deliveryMapper.getByMasterId(id);
    }

    @Override
    public Delivery getById(Integer storeId, Integer id) {
        return deliveryMapper.getById(storeId, id);
    }

    @Override
    public Pager<Delivery> list(DeliveryFilter filter, HttpServletRequest request) {
        HttpSession session = request.getSession();
        User user = JSON.parseObject(session.getAttribute(session.getId()).toString(), User.class);
        filter.setStoreId(user.getStoreId().intValue());
        filter.initPage();
        filter.initSort();
        Pager<Delivery> pager = new Pager<Delivery>(listCount(filter), filter.getPageNumber(), filter.getPageSize());
        List<Delivery> deliveries = deliveryMapper.getList(filter);
        deliveries.forEach(delivery -> delivery.setReturnSignStatusName(cn.bs.bsmanagebasic.utils.enums.DeliverySignReturnEnum.of(delivery.getReturnSignStatus())));
        pager.setList(deliveries);
        return pager;
    }

    @Override
    public Integer listCount(DeliveryFilter filter) {
        filter.initPage();
        return deliveryMapper.getListCount(filter);
    }

    /**
     * 订单处理，返回对应参数
     *
     * @param list 查询出来的订单集合
     * @param
     * @return 处理后的订单集合
     */
    private List<Delivery> returnDeliveryHandle(List<Delivery> list) {
        // storeId, expCode, expId, userPhone
        // 补充主表中没有的数据 key的格式： storeId_expId_expCode_userPhone
        Map<String, Delivery> completeData = new HashMap<>(14);
        if (null != list && list.size() > 0 && null == list.get(0).getSmsStatus()) {
            // 存储查询条件
            Map<Integer, List<Delivery>> param = new HashMap<>(14);
            // 根据门店将查询条件进行分组
            list.forEach(item -> {
                List<Delivery> deliveries;
                if (null == (deliveries = param.get(item.getStoreId()))) {
                    param.put(item.getStoreId(), new ArrayList<>());
                    deliveries = param.get(item.getStoreId());
                }
                deliveries.add(item);
            });

            param.forEach((Integer storeId, List<Delivery> list1) -> {
                // 根据总表的信息获取总表中不存在/没更新的数据
                deliveryMapper.slectByMaster(storeId, list1).stream().forEach(delivery -> {
                    String key = new StringBuilder(storeId)
                            .append("_").append(delivery.getExpId())
                            .append("_").append(delivery.getExpcode())
                            .append("_").append(delivery.getUserPhone()).toString();
                    completeData.put(key, delivery);
                });
            });
        }

        List<Delivery> result = list.stream().map(item -> {
            if (null != completeData && completeData.size() > 0) {
                String key = new StringBuilder(item.getStoreId())
                        .append("_").append(item.getExpId())
                        .append("_").append(item.getExpcode())
                        .append("_").append(item.getUserPhone()).toString();
                Delivery delivery = completeData.get(key);
                item.setSmsStatus(delivery.getSmsStatus());
                item.setSendSuccessType(delivery.getSendSuccessType());
                item.setReturnSignStatus(delivery.getReturnSignStatus());
            }

            Delivery jdo = JSON.parseObject(JSON.toJSONString(item), Delivery.class);
            // 判断是否有白名单用户
            if (null != item.getWhiteUser() && item.getWhiteUser().booleanValue()) {
                // 白名单用户状态为3，不发送短信
                jdo.setSmsStatus(3);
                jdo.setIsSendSMS(1);
            } else {
                jdo.setSmsStatus(null == item.getSmsStatus() ? -1 : item.getSmsStatus());
                if (item.getStatus() == DeliveryStatusConstants.ORDER_STATUS_STORED || item.getStatus() == DeliveryStatusConstants.ORDER_STATUS_REJECT_RETENTION) {
                    jdo.setIsSendSMS(0);
                } else {
                    jdo.setIsSendSMS(1);
                }
            }

            if (null != item.getResultInfo() && null != item.getSmsStatus() && 2 == item.getSmsStatus()) {
                jdo.setResultInfo(SMSErrorCodeEnum.of(item.getResultInfo()));
            } else {
                jdo.setResultInfo("");
            }
            if (null != item.getReturnSignStatus()) {
                jdo.setReturnSignStatusName(DeliverySignReturnEnum.of(item.getReturnSignStatus()));
            }
            if (null == item.getSendSuccessType()) {
                jdo.setSendSuccessType(0);
            } else {
                jdo.setSendSuccessType(item.getSendSuccessType());
            }

            String shelfName, shelfCode;
            shelfName = null == (shelfName = jdo.getShelfName()) ? "货架（无）" : shelfName;
            shelfCode = null == (shelfCode = jdo.getShelfCode()) ? "取件码（无）" : shelfCode;
            jdo.setShelfCode(new StringBuilder(shelfName).append("-").append(shelfCode).toString());

            if ((null != jdo.getStatus())) {
                jdo.setStatusName(DeliveryStatusEnum.of(jdo.getStatus().intValue()));
            }

            if (null != item.getSmsStatus()) {
                switch (item.getSmsStatus()) {
                    case 1:
                        jdo.setSmsStatusName("成功");
                        break;
                    case 2:
                        jdo.setSmsStatusName("失败");
                        break;
                    default:
                        jdo.setSmsStatusName("未通知");
                        break;
                }
            }

            if (null != item.getSendSuccessType()) {
                jdo.setSendSuccessTypeName(item.getSendSuccessType().compareTo(1) == 0 ? "短信" : "微信");
            }

            if (null != item.getStoreId()) {
                StoreDto dto = new StoreDto();
                dto.setId(item.getStoreId().longValue());
                StoreDto store = storeManageService.getStoreById(dto);
                if (null != store) {
                    jdo.setStoreName(store.getName());
                    jdo.setStoreAddress(new StringBuilder(store.getProvinceName()).append(store.getCityName()).append(store.getAreaName()).toString());
                }
            }

            return jdo;
        }).collect(Collectors.toList());
        return result;
    }

    @Override
    public List<String> getPickupCodeByPhone(String phone) {
        List<String> refunt = new ArrayList<>();
        DeliveryFilter filter = new DeliveryFilter();
        filter.setUserPhone(phone);
        LocalDateTime now = LocalDateTime.now();
        filter.setEndTime(now);
        filter.setStartTime(now.minus(30, ChronoUnit.DAYS));
        List<Delivery> deliveries = deliveryMapper.getPickupCodeByPhone(filter);
        if (deliveries != null) {
            deliveries.stream().collect(Collectors.groupingBy(Delivery::getStoreId)).entrySet().forEach(entry -> {
                StringJoiner sj = new StringJoiner(",", entry.getValue().get(0).getStoreName() + ":", "");
                entry.getValue().forEach(delivery -> {
                    sj.add(delivery.getShelfCode());
                });
                refunt.add(sj.toString());
            });
        } else {
            refunt.add("未查询您的快递信息");
        }
        return refunt;
    }
}
