package cn.bs.bsmanagebasic.service.impl;

import cn.bs.bsmanagebasic.common.constants.DeliveryStatusConstants;
import cn.bs.bsmanagebasic.common.constants.ExpressAccountConstants;
import cn.bs.bsmanagebasic.dao.jisu.ExpressAccountConsumeRecordMapper;
import cn.bs.bsmanagebasic.entity.ExpressAccountConsumeRecord;
import cn.bs.bsmanagebasic.entity.ExpressDeptAccount;
import cn.bs.bsmanagebasic.entity.delivery.Delivery;
import cn.bs.bsmanagebasic.entity.delivery.IdAndStoreIds;
import cn.bs.bsmanagebasic.entity.filter.DeliveryFilter;
import cn.bs.bsmanagebasic.entity.filter.ExpressDeptFilter;
import cn.bs.bsmanagebasic.service.DeliveryService;
import cn.bs.bsmanagebasic.service.ExpressDeptAccountConsumeService;
import cn.bs.bsmanagebasic.service.ExpressDeptAccountService;
import cn.bs.bsmanagebasic.service.ExpressDeptService;
import cn.bs.bsmanagecommon.exception.BusinessException;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.support.TransactionSynchronizationAdapter;
import org.springframework.transaction.support.TransactionSynchronizationManager;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * @author sunjj
 * @version 1.0.0
 * @ClassName ExpressDeptAccountConsumeServiceImpl.java
 * @Description TODO
 * @createTime 2019年12月02日 17:35:00
 */
@Service
@Slf4j
public class ExpressDeptAccountConsumeServiceImpl implements ExpressDeptAccountConsumeService {

    @Autowired
    private DeliveryService deliveryService;

    @Autowired
    private ExpressAccountConsumeRecordMapper consumeRecordMapper;

    @Autowired
    private ExpressDeptAccountService accountService;

    @Override
    public void updateExpStatus(DeliveryFilter filter) {
        List<ExpressAccountConsumeRecord> records = new ArrayList<>();
        // 变更为在库状态，给已经出库扣费的单子退钱
        if (DeliveryStatusConstants.ORDER_STATUS_STORED == filter.getUpdateStatus()) {
            for (IdAndStoreIds idAndStoreId : filter.getIdAndStoreIds()) {
                Delivery delivery = new Delivery();
                BeanUtils.copyProperties(idAndStoreId, delivery);
                delivery.setExpcode(idAndStoreId.getExpCode());
                // 获取总表ID
                Integer masterId = deliveryService.queryIdFromMasterDelivery(delivery);
                // 根据快递公司ID和总表ID查询消费记录
                ExpressAccountConsumeRecord record = consumeRecordMapper.selectLastRecord(masterId, idAndStoreId.getExpId());
                // 如果查询到消费记录,并且是出库扣费的记录，插入一条退款记录
                if (record != null && record.getDeductionNode().byteValue() == ExpressAccountConstants.DEDUCTION_NODE_COMPLETE) {
                    // 退款记录
                    record.setType(ExpressAccountConstants.CONSUME_TYPE_REFUND);
                    record.setRemark("修改运单状态退款 - CONSUME_ID:" + record.getId());
                    records.add(record);
                }
            }
        } else if (DeliveryStatusConstants.ORDER_STATUS_COMPLETE == filter.getUpdateStatus()) {
            // 变更为出库状态 扣费
            for (IdAndStoreIds idAndStoreId : filter.getIdAndStoreIds()) {
                ExpressDeptAccount account = accountService.getAccount(idAndStoreId.getStoreId().longValue(), idAndStoreId.getExpId().longValue());
                // 如果账号是出库扣费，插入一条扣费记录
                if (null != account && account.getDeductionNode().compareTo(ExpressAccountConstants.DEDUCTION_NODE_COMPLETE) == 0) {
                    Delivery delivery = new Delivery();
                    BeanUtils.copyProperties(idAndStoreId, delivery);
                    delivery.setExpcode(idAndStoreId.getExpCode());
                    // 获取总表ID
                    Integer masterId = deliveryService.queryIdFromMasterDelivery(delivery);
                    // 插入一条扣费记录
                    if (null != masterId) {
                        delivery.setId(masterId);
                        records.add(createRecord(delivery, account));
                    }
                }
            }
        }
        // 批量插入数据
        if (null != records && records.size() > 0) {
            consumeRecordMapper.insertList(records);
        }

    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateExpressAccountAmount() {
        // 1.获取要统计数据的开始时间和结束时间
        ExpressAccountConsumeRecord recordTime = consumeRecordMapper.selectTime();
        ExpressDeptFilter filter = new ExpressDeptFilter();
        if (null == recordTime) {
            throw new BusinessException("获取时间异常");
        }
        if (null != recordTime.getFinishTime()) {
            filter.setCreateBeginTime(recordTime.getFinishTime());
        }
        if (null != recordTime && null != recordTime.getCreateTime()) {
            filter.setCreateEndTime(recordTime.getCreateTime());
        }

        filter.setCount(0L);
        filter.setPageSize(1000);
        // 扣费计算存储
        Map<String, BigDecimal> deductionMap = new HashMap<>();
        // 统计条数计算存储
        Map<String, AtomicInteger> countMap = new HashMap<>();
        while (true) {
            // 获取需要统计的数据
            List<ExpressAccountConsumeRecord> list = consumeRecordMapper.selectForStatisticsByFilter(filter);

            for (ExpressAccountConsumeRecord record : list) {
                String key = record.getStoreId() + "_" + record.getExpId();
                if (null == countMap.get(key)) {
                    countMap.put(key, new AtomicInteger(1));
                } else {
                    countMap.get(key).incrementAndGet();
                }

                if (null == deductionMap.get(key)) {
                    deductionMap.put(key, new BigDecimal(0));
                }
                BigDecimal deduction = deductionMap.get(key);
                switch (record.getType()) {
                    case ExpressAccountConstants.CONSUME_TYPE_DEDUCTION:
                        deductionMap.put(key, deduction.add(record.getPackagePrice()));
                        break;
                    case ExpressAccountConstants.CONSUME_TYPE_REFUND:
                        deductionMap.put(key, deduction.subtract(record.getPackagePrice()));
                        break;
                    default:
                        break;
                }
            }

            if (list.size() < filter.getPageSize()) {
                break;
            }
            filter.setCount(filter.getCount() + filter.getPageSize());
        }

        // 统计数据
        List<ExpressAccountConsumeRecord> insertRecords = new ArrayList<>();
        deductionMap.forEach((String key, BigDecimal value) -> {
            String[] splitKey = key.split("_");
            Long storeId = Long.valueOf(splitKey[0]);
            Long expId = Long.valueOf(splitKey[1]);
            ExpressAccountConsumeRecord record = new ExpressAccountConsumeRecord();
            record.setStoreId(storeId);
            record.setExpId(expId);
            record.setPackagePrice(value);
            record.setFinishTime(filter.getCreateEndTime());
            record.setType(ExpressAccountConstants.CONSUME_TYPE_MERGE);
            record.setRemark("定时任务统计扣费");
            record.setPackageCount(countMap.get(key).get());

            int retry = 2;
            int result = 0;
            while (result == 0) {
                if (retry == 0) {
                    log.warn("更新余额失败 storeId: [}, expId: {}", storeId, expId);
                    break;
                }
                ExpressDeptAccount account = accountService.selectAccountByStoreIdAndExpId(storeId, expId);
                if (null == account) {
                    break;
                }
                record.setAccountId(account.getId());
                ExpressDeptAccount accountForUpdate = new ExpressDeptAccount();
                accountForUpdate.setId(account.getId());
                accountForUpdate.setVersion(account.getVersion());
                accountForUpdate.setStoreId(account.getStoreId());
                accountForUpdate.setExpId(account.getExpId());
                BigDecimal balance = account.getBalance();
                if (null == balance) {
                    balance = new BigDecimal(0);
                }
                accountForUpdate.setBalance(balance.subtract(value));
                result = accountService.updateBalance(accountForUpdate, true);
                retry--;
            }
            insertRecords.add(record);
        });

        if (null == insertRecords || insertRecords.size() == 0) {
            ExpressAccountConsumeRecord record = new ExpressAccountConsumeRecord();
            record.setPackagePrice(new BigDecimal(0));
            record.setFinishTime(filter.getCreateEndTime());
            record.setType(ExpressAccountConstants.CONSUME_TYPE_MERGE);
            record.setRemark("定时任务统计扣费");
            record.setPackageCount(0);
            insertRecords.add(record);
        }
        consumeRecordMapper.insertList(insertRecords);

    }

    @Override
    public boolean updateExpressAccountAmountLock() {
        int i = consumeRecordMapper.deleteForLock();
        return i > 0;
    }

    @Override
    public boolean updateExpressAccountAmountUNLock() {
        int i = consumeRecordMapper.insertForUNLock();
        return i > 0;
    }


    private ExpressAccountConsumeRecord createRecord(Delivery delivery, ExpressDeptAccount account) {
        ExpressAccountConsumeRecord record = new ExpressAccountConsumeRecord();
        record.setStoreId(delivery.getStoreId().longValue());
        record.setExpId(delivery.getExpId().longValue());
        record.setDeliveryId(delivery.getId().longValue());
        record.setExpCode(delivery.getExpcode());

        record.setAccountId(account.getId());
        record.setDeductionNode(account.getDeductionNode());
        record.setPackageCount(1);
        record.setPackagePrice(account.getPackagePrice());
        record.setType(ExpressAccountConstants.CONSUME_TYPE_DEDUCTION);
        return record;
    }
}
