package cn.bs.bsmanagebasic.service.impl;

import cn.bs.bsmanagebasic.common.util.CacheTool;
import cn.bs.bsmanagebasic.dao.jeesite.ExpressDeptAccountMapper;
import cn.bs.bsmanagebasic.entity.ExpressDeptAccount;
import cn.bs.bsmanagebasic.service.ExpressDeptAccountService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionSynchronizationAdapter;
import org.springframework.transaction.support.TransactionSynchronizationManager;

import java.util.List;

/**
 * @author sunjj
 * @version 1.0.0
 * @ClassName ExpressDeptAccountServiceImpl.java
 * @Description TODO
 * @createTime 2019年12月04日 17:09:00
 */
@Service
public class ExpressDeptAccountServiceImpl implements ExpressDeptAccountService {
    @Autowired
    private ExpressDeptAccountMapper mapper;

    @Override
    public ExpressDeptAccount getAccount(Long storeId, Long expId) {
        {
            CacheTool cacheTool = CacheTool.initExpressAccountTool();
            ExpressDeptAccount account = (ExpressDeptAccount) cacheTool.getBean(storeId.toString() + expId);
            if (null == account) {
                account = this.selectAccountByStoreIdAndExpId(storeId, expId);
                if (null == account) {
                    account = new ExpressDeptAccount();
                }
                cacheTool.saveBean(generatorRedisKey(account.getStoreId(), account.getExpId()), account);
            }
            return null == account.getId() ? null : account;
        }
    }

    @Override
    public int updateByPrimaryKeySelective(ExpressDeptAccount account) {
        int result = mapper.updateByPrimaryKeySelective(account);
        upDateCacheByAccountId(account.getId());
        return result;
    }

    @Override
    public void upDateCacheByDeptId(Long id) {
        List<ExpressDeptAccount> list = mapper.selectByDeptId(id);
        if (null != list && list.size() > 0) {
            // 更新缓存
            list.stream().forEach(account -> upDateCache(account));
        }
    }

    @Override
    public int enableAccountById(Long id) {
        Integer result = mapper.enableAccountById(id);
        if (result > 0) {
            upDateCacheByAccountId(id);
        }
        return result;
    }

    @Override
    public int deleteAccountById(Long id) {

        Integer result = mapper.deleteAccountById(id);
        if (result > 0) {
            upDateCacheByAccountId(id);
        }
        return result;
    }

    @Override
    public ExpressDeptAccount selectAccountByStoreIdAndExpId(Long storeId, Long expId) {
        return mapper.selectAccountByStoreIdAndExpId(storeId, expId);
    }

    @Override
    public int updateBalance(ExpressDeptAccount accountForUpdate, boolean isTransaction) {
        int i = mapper.updateBalance(accountForUpdate);
        if (i > 0) {
            if (isTransaction) {
                // 开启了事务，在事务提交后更新Redis
                TransactionSynchronizationManager.registerSynchronization(new TransactionSynchronizationAdapter() {
                    @Override
                    public void afterCommit() {
                        if (null != accountForUpdate.getStoreId() && null != accountForUpdate.getExpId()) {
                            upDateCache(accountForUpdate);
                        } else {
                            upDateCacheByAccountId(accountForUpdate.getId());
                        }
                    }
                });
            } else {
                upDateCacheByAccountId(accountForUpdate.getId());
            }
        }
        return i;
    }

    private void upDateCacheByAccountId(Long id) {
        ExpressDeptAccount account = mapper.selectAccountById(id);
        if (null != account) {
            upDateCache(account);
        }
    }

    private void upDateCache(ExpressDeptAccount account) {
        CacheTool cacheTool = CacheTool.initExpressAccountTool();
        cacheTool.hdel(generatorRedisKey(account.getStoreId(), account.getExpId()));
    }

    private String generatorRedisKey(Long storeId, Long expId) {
        return new StringBuilder().append(storeId).append("_").append(expId).toString();
    }
}
