package cn.bs.bsmanagebasic.service.impl;

import cn.bs.bsmanagebasic.common.util.Pager;
import cn.bs.bsmanagebasic.dao.jeesite.ExpressDeptAccountMapper;
import cn.bs.bsmanagebasic.dao.jeesite.ExpressDeptMapper;
import cn.bs.bsmanagebasic.dao.jisu.ExpressAccountConsumeRecordMapper;
import cn.bs.bsmanagebasic.dao.jisu.ExpressAccountRechargeRecordMapper;
import cn.bs.bsmanagebasic.entity.ExExpressType;
import cn.bs.bsmanagebasic.entity.ExpressAccountRechargeRecord;
import cn.bs.bsmanagebasic.entity.ExpressDept;
import cn.bs.bsmanagebasic.entity.ExpressDeptAccount;
import cn.bs.bsmanagebasic.entity.dto.ExpressAccountConsumeRecordDto;
import cn.bs.bsmanagebasic.entity.dto.ExpressAccountRechargeRecordDto;
import cn.bs.bsmanagebasic.entity.dto.StoreDto;
import cn.bs.bsmanagebasic.entity.filter.ExpressDeptFilter;
import cn.bs.bsmanagebasic.service.ExpressDeptAccountService;
import cn.bs.bsmanagebasic.service.ExpressDeptService;
import cn.bs.bsmanagebasic.service.ExpressService;
import cn.bs.bsmanagebasic.service.StoreManageService;
import cn.bs.bsmanagecommon.exception.BusinessException;
import cn.bs.bsmanagecommon.util.DateUtil;
import org.apache.logging.log4j.util.Strings;
import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author sunjj
 * @version 1.0.0
 * @ClassName ExpressDeptServiceImpl.java
 * @Description TODO
 * @createTime 2019年11月18日 10:53:00
 */
@Service
public class ExpressDeptServiceImpl implements ExpressDeptService {
    @Autowired
    private ExpressDeptMapper deptMapper;

    @Autowired
    private ExpressDeptAccountMapper deptAccountMapper;

    @Autowired
    private ExpressAccountRechargeRecordMapper recordMapper;

    @Autowired
    private ExpressService expressService;

    @Autowired
    private StoreManageService storeManageService;

    @Autowired
    private ExpressAccountConsumeRecordMapper consumeRecordMapper;

    @Autowired
    private ExpressDeptAccountService accountService;

    /**
     * @throws
     * @description 添加快递部门（对应画面快递账号，账号增加）
     * @author sunjj
     * @updateTime
     */
    @Override
    public void addDept(ExpressDept dept) {
        ExExpressType express = expressService.getExpressById(dept.getExpId());
        if (null == express) {
            throw new BusinessException("所选的快递公司不存在");
        }
        dept.setExpressName(express.getName());
        generatorAccount(dept);
        deptMapper.insertSelective(dept);
    }

    /**
     * @throws
     * @description 获取快递部门列表（对应画面快递账号列表）
     * @author sunjj
     * @updateTime
     */
    @Override
    public Pager<ExpressDept> getDepts(ExpressDeptFilter filter) {
        filter.initPage();
        if (null != filter.getCreateEndTime()) {
            // 结束时间+1天
            filter.setCreateEndTime(new DateTime(filter.getCreateEndTime()).plusDays(1).toDate());
        }
        Long total = deptMapper.countByFilter(filter);
        Pager<ExpressDept> page = new Pager<>(total, filter.getPageNumber(), filter.getPageSize());
        List<ExpressDept> depts;
        if (null != total && total.compareTo(0L) > 0) {
            depts = deptMapper.selectByFilter(filter);
        } else {
            depts = new ArrayList<>();
        }
        page.setList(depts);
        return page;
    }

    /**
     * @throws
     * @description 编辑快递部门（画面账号编辑）
     * @author sunjj
     * @updateTime
     */
    @Override
    public void editDept(ExpressDept dept) {
        if (null != dept.getExpId()) {
            // 校验, 当前账号已经拥有账户时不允许修改快递公司
            ExpressDept expressDept = deptMapper.selectByPrimaryKey(dept.getId());
            Long count = deptAccountMapper.countByDeptId(dept.getId());

            if (expressDept.getExpId().compareTo(dept.getExpId()) != 0 && count != null && count.compareTo(0L) > 0) {
                throw new BusinessException("当前已有关联账户，请不要随意修改快递公司！");
            }

            ExExpressType express = expressService.getExpressById(dept.getExpId());

            dept.setExpressName(express.getName());
            if (Strings.isBlank(dept.getDeptName())) {
                dept.setDeptName(expressDept.getDeptName());
            }
            generatorAccount(dept);
        }
        deptMapper.updateByPrimaryKeySelective(dept);
    }

    /**
     * @throws
     * @description 创建快递账户（一个快递账号+一个门店对应一个账户/ 另一个快递公司只能绑定一个门店 ）
     * @author sunjj
     * @updateTime
     */
    @Override
    public void addDeptAccount(ExpressDeptAccount account) {
        ExpressDeptFilter filter = new ExpressDeptFilter();
        filter.setStoreId(account.getStoreId());
        filter.setExpId(account.getExpId());
        Long count = deptAccountMapper.countByFilter(filter);
        if (null != count && count.intValue() > 0) {
            throw new BusinessException("请勿重复开户");
        }

        ExExpressType express = expressService.getExpressById(account.getExpId());
        account.setExpressName(express.getName());

        StoreDto dto = new StoreDto();
        dto.setId(account.getStoreId());
        StoreDto store = storeManageService.getStoreById(dto);
        if (null == store) {
            throw new BusinessException("所选门店不存在");
        }
        account.setStoreName(store.getName());

        ExpressDept expressDept = deptMapper.selectByPrimaryKey(account.getExpressDeptId());
        if (null == expressDept) {
            throw new BusinessException("所选快递账号不存在");
        }
        account.setAccount(store.getId() + expressDept.getAccount());
        try {
            deptAccountMapper.insertSelective(account);
        } catch (DuplicateKeyException exception) {
            throw new BusinessException("快递账号和门店重复绑定");
        }
    }

    /**
     * @throws
     * @throws
     * @description 快递账户，账户列表
     * @author sunjj
     * @updateTime
     */
    @Override
    public Pager<ExpressDeptAccount> getDeptAccounts(ExpressDeptFilter filter) {
        filter.initPage();
        Long total = deptAccountMapper.countByFilter(filter);
        Pager<ExpressDeptAccount> page = new Pager<>(total, filter.getPageNumber(), filter.getPageSize());
        List<ExpressDeptAccount> deptAccounts;
        if (null != total && total.compareTo(0L) > 0) {
            deptAccounts = deptAccountMapper.selectByFilter(filter);
        } else {
            deptAccounts = new ArrayList<>();
        }
        page.setList(deptAccounts);
        return page;
    }

    /**
     * @throws
     * @description 快递账户-账户编辑
     * @author sunjj
     * @updateTime
     */
    @Override
    public void editDeptAccount(ExpressDeptAccount account) {
        // 处理禁止更新的选项
        updateDisableProcess(account);
        try {
            accountService.updateByPrimaryKeySelective(account);
        } catch (DuplicateKeyException exception) {
            throw new BusinessException("快递账号和门店重复绑定");
        }
    }

    /**
     * @throws
     * @description 快递公司账户不允许更新内容处理
     * @author sunjj
     * @updateTime
     */
    private void updateDisableProcess(ExpressDeptAccount account) {
        // 快递账户,门店,快递公司禁止更新
        account.setStoreId(null);
        account.setStoreName(null);
        account.setAccount(null);
    }

    @Override
    public List<ExpressDept> getDeptsForOption(Long expId) {
        return deptMapper.selectForOption(expId);
    }

    @Override
    public ExpressDept getDeptDetail(ExpressDeptFilter filter) {
        return deptMapper.selectByPrimaryKey(filter.getId());
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void expressAccountRecharge(ExpressDeptFilter filter) {
        BigDecimal amount = new BigDecimal(filter.getAmount());
        int result = 0;
        int retry = 3;
        while (result == 0) {
            if (retry == 0) {
                throw new BusinessException("系统异常请稍后重试");
            }
            ExpressDeptAccount expressDeptAccount = deptAccountMapper.selectByPrimaryKey(filter.getId());
            if (expressDeptAccount == null) {
                throw new BusinessException("充值账号不存在");
            }
            // 更新余额
            BigDecimal balance = expressDeptAccount.getBalance();
            BigDecimal newBalance = balance.add(amount);
            ExpressDeptAccount account = new ExpressDeptAccount();
            account.setId(filter.getId());
            account.setBalance(newBalance);
            account.setVersion(expressDeptAccount.getVersion());
            result = accountService.updateBalance(account, true);
            retry--;
        }

        // 保存充值记录
        ExpressAccountRechargeRecord record = new ExpressAccountRechargeRecord();
        record.setAmount(amount);
        record.setAccountId(filter.getId());
        record.setPayStatus(1);
        record.setPayWay(4);
        // TODO 没有登录功能， 使用固定值 admin
        record.setOperator("admin");
        if (recordMapper.insertSelective(record) == 0) {
            throw new BusinessException("系统异常请稍后重试");
        }
    }

    @Override
    public boolean deleteById(Long id) {
        if (id == null) {
            throw new BusinessException("传入参数不能为空");
        }
        accountService.upDateCacheByDeptId(id);
        boolean result = deptMapper.deleteById(id) > 0;
        return result;
    }

    @Override
    public boolean save(ExpressDept dept) {
        if (dept.getId() == null) {
            addDept(dept);
        } else {
            editDept(dept);
        }
        return true;
    }

    @Override
    public boolean saveAcount(ExpressDeptAccount deptAcount) {
        if (deptAcount.getId() == null) {
            addDeptAccount(deptAcount);
        } else {
            editDeptAccount(deptAcount);
        }
        return true;
    }

    @Override
    public boolean enableById(Long id) {
        if (id == null) {
            throw new BusinessException("传入参数不能为空");
        }
        boolean result = deptMapper.enableById(id) > 0;
        if (result) {
            accountService.upDateCacheByDeptId(id);
        }
        return result;
    }

    @Override
    public boolean enableAccountById(Long id) {
        if (id == null) {
            throw new BusinessException("传入参数不能为空");
        }
        return accountService.enableAccountById(id) > 0;
    }

    @Override
    public boolean deleteAccountById(Long id) {
        if (id == null) {
            throw new BusinessException("传入参数不能为空");
        }
        boolean result = accountService.deleteAccountById(id) > 0;
        return result;
    }

    @Override
    public Pager<ExpressAccountRechargeRecordDto> getRecords(ExpressDeptFilter filter) {
        filter.initPage();
        if (null != filter.getCreateEndTime()) {
            // 结束时间+1天
            filter.setCreateEndTime(new DateTime(filter.getCreateEndTime()).plusDays(1).toDate());
        }
        Long total = recordMapper.countByFilter(filter);
        Pager<ExpressAccountRechargeRecordDto> page = new Pager<>(total, filter.getPageNumber(), filter.getPageSize());
        List<ExpressAccountRechargeRecordDto> depts;
        if (null != total && total.compareTo(0L) > 0) {
            depts = recordMapper.selectByFilter(filter);
        } else {
            depts = new ArrayList<>();
        }
        page.setList(depts);
        return page;
    }

    @Override
    public Pager<ExpressAccountConsumeRecordDto> getConsumeRecords(ExpressDeptFilter filter) {
        filter.initPage();
        if (null != filter.getCreateEndTime()) {
            // 结束时间+1天
            filter.setCreateEndTime(new DateTime(filter.getCreateEndTime()).plusDays(1).toDate());
        }
        Long total = consumeRecordMapper.countByFilter(filter);
        Pager<ExpressAccountConsumeRecordDto> page = new Pager<>(total, filter.getPageNumber(), filter.getPageSize());
        if (null != total || total.compareTo(0L) > 0) {
            List<ExpressAccountConsumeRecordDto> expressAccountConsumeRecordDtos = consumeRecordMapper.selectByFilter(filter);
            page.setList(expressAccountConsumeRecordDtos);
        }
        return page;
    }

    @Override
    public Pager<ExpressAccountConsumeRecordDto> getDeliveries(ExpressDeptFilter filter) {
        filterHandler(filter);
        Long total = consumeRecordMapper.countForDeliveryByFilter(filter);
        Pager<ExpressAccountConsumeRecordDto> pager = new Pager(total, filter.getPageNumber(), filter.getPageSize());
        if (null != total && total.compareTo(0L) > 0) {
            List<ExpressAccountConsumeRecordDto> deliveries = consumeRecordMapper.selectForDeliveryByFilter(filter);
            pager.setList(deliveries);
        }
        return pager;
    }

    private void generatorAccount(ExpressDept dept) {
        StringBuilder builder = new StringBuilder(dept.getExpId().toString());
        if (!Strings.isBlank(dept.getExpressName())) {
            builder.append(String.format("%02d", dept.getExpressName().hashCode() & 99));
        }
        if (!Strings.isBlank(dept.getDeptName())) {
            builder.append(String.format("%03d", dept.getDeptName().hashCode() & 999));
        }
        dept.setAccount(builder.toString());
    }

    private void filterHandler(ExpressDeptFilter filter) {
        filter.initPage();

        if (null == filter.getCreateBeginTime()) {
            filter.setCreateBeginTime(DateUtil.getFirstDayOfMonth());
        }

        if (null != filter.getCreateEndTime()) {
            Date date = DateUtil.addDay(filter.getCreateEndTime(), 1);
            if (DateUtil.compareNow(date) >= 0) {
                date = null;
            }
            filter.setCreateEndTime(date);
        }
    }

}
