package cn.bs.bsmanagebasic.service.impl;

import cn.bs.bsmanagebasic.entity.oss.*;
import cn.bs.bsmanagebasic.service.ConfigurationService;
import cn.bs.bsmanagebasic.service.FileManagementService;
import cn.bs.bsmanagecommon.exception.BusinessException;
import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClientBuilder;
import com.aliyun.oss.model.*;
import org.apache.logging.log4j.util.Strings;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.net.URL;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @packageName: cn.bs.bsmanagebasic.service.impl
 * @className: bs-manage
 * @description: OSS文件管理业务
 * @author: liujs
 * @date: 2019/12/16 0016 下午 14:40
 * @version: 1.0
 */
@Service
public class FileManagementServiceImpl implements FileManagementService {
    @Resource
    private ConfigurationService configurationService;

    @Override
    public String addBucket(BucketFilter bucketFilter) {
        try {
            OssClient ossClient = new OssClient().invoke(configurationService);
            //创建实例
            OSS oss = new OSSClientBuilder().build(ossClient.getEndpoint(),ossClient.getAccessKeyId(),ossClient.getAccessKeySecret());
            //创建CreateBucketRequest对象
            CreateBucketRequest createBucketRequest = new CreateBucketRequest(bucketFilter.getBucketName());
            // 设置存储类型(默认是标准类型)
            if (Strings.isBlank(bucketFilter.getStorageType())) {
                bucketFilter.setStorageType("Standard");
            }
            createBucketRequest.setStorageClass(StorageClass.valueOf(bucketFilter.getStorageType()));
            //设置数据容灾类型(默认是本地容灾类型)
            if (Strings.isBlank(bucketFilter.getDataRedundancyType())) {
                bucketFilter.setDataRedundancyType("LRS");
            }
            createBucketRequest.setDataRedundancyType(DataRedundancyType.valueOf(bucketFilter.getDataRedundancyType()));
            //设置存储空间的权限
            if (Strings.isBlank(bucketFilter.getCannedAccessControlList())){
                bucketFilter.setCannedAccessControlList("Private");
            }
            createBucketRequest.setCannedACL(CannedAccessControlList.valueOf(bucketFilter.getCannedAccessControlList()));
            //创建存储空间
            oss.createBucket(createBucketRequest);
            oss.shutdown();
        }catch (Exception e){
            throw new BusinessException("参数错误,创建失败");
        }
        return "创建成功";
    }

    @Override
    public Boolean deleteBucket(BucketFilter bucketFilter) {
        try {
            OssClient ossClient = new OssClient().invoke(configurationService);
            OSS oss = new OSSClientBuilder().build(ossClient.getEndpoint(),ossClient.getAccessKeyId(),ossClient.getAccessKeySecret());
            oss.deleteBucket(bucketFilter.getBucketName());
            oss.shutdown();
        }catch (Exception e){
            throw new BusinessException("参数错误,删除失败");
        }
        return true;
    }

    @Override
    public List<String> bucketList(BucketFilter bucketFilter) {
        List<String> bucketList = new ArrayList<>();
        switch (bucketFilter.getSelectBucketType()){
            case 0:
                bucketList = selectAllBucket();
                break;
            case 1:
                bucketList = selectPrefixBucket(bucketFilter);
                break;
            case 2:
                bucketList = selectMarkerBucket(bucketFilter);
                break;
            case 3:
                bucketList = selectNumberBucket(bucketFilter);
                break;
            default:
                break;
        }
        return bucketList;
    }

    @Override
    public BucketInfo bucketInfo(BucketFilter bucketFilter) {
        BucketInfo info = null;
        try {
            OssClient ossClient = new OssClient().invoke(configurationService);
            //创建实例
            OSS oss = new OSSClientBuilder().build(ossClient.getEndpoint(),ossClient.getAccessKeyId(),ossClient.getAccessKeySecret());
            // 存储空间的信息包括地域（Region或Location）、创建日期（CreationDate）、拥有者（Owner）、权限（Grants）等。
            info = oss.getBucketInfo(bucketFilter.getBucketName());
            // 关闭OSSClient。
            oss.shutdown();
        }catch (Exception e){
            throw new BusinessException("不存在该存储空间");
        }
        return info;
    }

    @Override
    public Boolean isExistBucket(BucketFilter bucketFilter) {
        Boolean exists = false;
        try {
            OssClient ossClient = new OssClient().invoke(configurationService);
            //创建实例
            OSS oss = new OSSClientBuilder().build(ossClient.getEndpoint(),ossClient.getAccessKeyId(),ossClient.getAccessKeySecret());
             exists = oss.doesBucketExist(bucketFilter.getBucketName());
            // 关闭OSSClient。
            oss.shutdown();
        }catch (Exception e){
            throw new BusinessException("发生异常");
        }
        return exists;
    }

    @Override
    public Boolean updateBucket(BucketFilter bucketFilter) {
        try {
            OssClient ossClient = new OssClient().invoke(configurationService);
            //创建实例
            OSS oss = new OSSClientBuilder().build(ossClient.getEndpoint(),ossClient.getAccessKeyId(),ossClient.getAccessKeySecret());
            // 设置存储空间的访问权限为私有。
            oss.setBucketAcl(bucketFilter.getBucketName(), CannedAccessControlList.valueOf((bucketFilter.getCannedAccessControlList())));
            // 关闭OSSClient。
            oss.shutdown();
        }catch (Exception e){
            throw new BusinessException("参数错误,修改失败");
        }
        return true;
    }

    @Override
    public Boolean uploadFile(BucketFilter bucketFilter) {
        try {
            OssClient ossClient = new OssClient().invoke(configurationService);
            //创建实例
            OSS oss = new OSSClientBuilder().build(ossClient.getEndpoint(),ossClient.getAccessKeyId(),ossClient.getAccessKeySecret());
            //上传文件(新增文件)
            PutObjectRequest putObjectRequest = null;
            if (Strings.isNotBlank(bucketFilter.getObjectName())) {
                // 创建PutObjectRequest对象。
                 putObjectRequest = new PutObjectRequest(bucketFilter.getBucketName(), bucketFilter.getObjectName(), new File(bucketFilter.getFilePath()));
            }
            //上传文件夹(新建文件夹)
            if (Strings.isNotBlank(bucketFilter.getFolderName())){
                // 创建PutObjectRequest对象。
                 putObjectRequest = new PutObjectRequest(bucketFilter.getBucketName(), bucketFilter.getFolderName()+"/", new ByteArrayInputStream(new byte[0]));
            }
            // 上传文件。(带有进度条的上传)
            oss.putObject(putObjectRequest.<PutObjectRequest>withProgressListener(new PutObjectProgressListener()));
            /*oss.putObject(putObjectRequest);*/
            // 关闭OSSClient。
            oss.shutdown();
        }catch (Exception e){
            throw new BusinessException("创建失败");
        }
        return true;
    }

    @Override
    public Boolean downloadFile(BucketFilter bucketFilter) {
        try{
            OssClient ossClient = new OssClient().invoke(configurationService);
            //创建实例
            OSS oss = new OSSClientBuilder().build(ossClient.getEndpoint(),ossClient.getAccessKeyId(),ossClient.getAccessKeySecret());
            // 下载OSS文件到本地文件。如果指定的本地文件存在会覆盖，不存在则新建。(带有进度条的下载)
            oss.getObject(new GetObjectRequest(bucketFilter.getBucketName(), bucketFilter.getObjectName()).<GetObjectRequest>withProgressListener(new GetObjectProgressListener()), new File(bucketFilter.getFilePath()));

            // 关闭OSSClient。
            oss.shutdown();
        }catch (Exception e){
            throw new BusinessException("下载失败");
        }
        return true;
    }

    @Override
    public List<String> deleteFile(BucketFilter bucketFilter) {
        List<String> deleteObjectList = null;
        try {
            OssClient ossClient = new OssClient().invoke(configurationService);
            OSS oss = new OSSClientBuilder().build(ossClient.getEndpoint(),ossClient.getAccessKeyId(),ossClient.getAccessKeySecret());
            //批量删除文件
            DeleteObjectsResult deleteObjectsResult = oss.deleteObjects(new DeleteObjectsRequest(bucketFilter.getBucketName()).withKeys(bucketFilter.getObjectNameList()));
            deleteObjectList = deleteObjectsResult.getDeletedObjects();
            oss.shutdown();
        }catch (Exception e){
            throw new BusinessException("删除失败");
        }
        return deleteObjectList;
    }

    @Override
    public OssFileInfoDto fileInfo(BucketFilter bucketFilter) {
        OssFileInfoDto ossFileInfoDto = new OssFileInfoDto();
        try{
            OssClient ossClient = new OssClient().invoke(configurationService);
            OSS oss = new OSSClientBuilder().build(ossClient.getEndpoint(),ossClient.getAccessKeyId(),ossClient.getAccessKeySecret());
            ossFileInfoDto.setObjectName(bucketFilter.getObjectName());

            ObjectMetadata objectMetadata = oss.getObjectMetadata(bucketFilter.getBucketName(),bucketFilter.getObjectName());

            ossFileInfoDto.setFileType(objectMetadata.getContentType());
            ossFileInfoDto.setFileStorageType(objectMetadata.getObjectStorageClass().toString());
            ossFileInfoDto.setETag(objectMetadata.getETag());
            ossFileInfoDto.setUpdateTime(new SimpleDateFormat("yyyy-MM-dd HH:mm").format(objectMetadata.getLastModified()));
            //计算单个文件大小
            String size = calculateObjectLength(oss, bucketFilter);
            ossFileInfoDto.setFileSize(size);

            ObjectAcl objectAcl = oss.getObjectAcl(bucketFilter.getBucketName(),bucketFilter.getObjectName());
            ossFileInfoDto.setFileACL(objectAcl.getPermission().toString());

            Date expiration = new Date(System.currentTimeMillis() + 3600L * 1000 * 24 * 180);
            URL url = oss.generatePresignedUrl(bucketFilter.getBucketName(),bucketFilter.getObjectName(),expiration);
            String url1 = url.toString().substring(0,url.toString().indexOf("?"));
            ossFileInfoDto.setUrl(url1);

            oss.shutdown();
        }catch (Exception e){
            throw new BusinessException("出现错误,获取信息失败");
        }
        return ossFileInfoDto;
    }

    private String calculateObjectLength(OSS oss, BucketFilter bucketFilter) {
        ListObjectsRequest request = new ListObjectsRequest(bucketFilter.getBucketName()).withPrefix(bucketFilter.getObjectName());
        List<OSSObjectSummary> sum = oss.listObjects(request).getObjectSummaries();
        //对数值进行处理之后返回
        return dealWithObjectLength(sum.get(0).getSize());
    }

    private String dealWithObjectLength(Long size) {
        DecimalFormat df = new DecimalFormat("#.00");
        String fileSizeString = "";
        String wrongSize = "0B";
        if (size == 0) {
            return wrongSize;
        }
        if (size < 1024) {
            fileSizeString = df.format((double) size) + "B";
        } else if (size < 1048576) {
            fileSizeString = df.format((double) size / 1024) + "KB";
        } else if (size < 1073741824) {
            fileSizeString = df.format((double) size / 1048576) + "MB";
        } else {
            fileSizeString = df.format((double) size / 1073741824) + "GB";
        }
        return fileSizeString;
    }

    @Override
    public Boolean updateFile(BucketFilter bucketFilter) {
        try {
            OssClient ossClient = new OssClient().invoke(configurationService);
            OSS oss = new OSSClientBuilder().build(ossClient.getEndpoint(),ossClient.getAccessKeyId(),ossClient.getAccessKeySecret());
            //修改文件存储类型
            ObjectMetadata objectMetadata = oss.getObjectMetadata(bucketFilter.getBucketName(),bucketFilter.getObjectName());
            if (!Strings.isBlank(bucketFilter.getStorageType()) && !objectMetadata.getObjectStorageClass().toString().equals(bucketFilter.getStorageType())){
                switch(objectMetadata.getObjectStorageClass().toString()){
                    case "Standard":
                        if("IA".equals(bucketFilter.getStorageType())){
                            standardToIA(bucketFilter,oss);
                        }else if("Archive".equals(bucketFilter.getStorageType())){
                            standardToArchive(bucketFilter,oss);
                        }else {
                            break;
                        }
                        break;
                    case "IA":
                        if("Standard".equals(bucketFilter.getStorageType())){
                            iAToStandard(bucketFilter,oss);
                        }else if("Archive".equals(bucketFilter.getStorageType())){
                            iAToArchive(bucketFilter,oss);
                        }else {
                            break;
                        }
                        break;
                    case "Archive":
                        if("Standard".equals(bucketFilter.getStorageType())){
                            archiveToStandard(bucketFilter,oss);
                        }else if("IA".equals(bucketFilter.getStorageType())){
                            archiveToIA(bucketFilter,oss);
                        }else {
                            break;
                        }
                        break;
                    default:
                        break;
                }
            }
            //修改读写类型
            ObjectAcl objectAcl = oss.getObjectAcl(bucketFilter.getBucketName(),bucketFilter.getObjectName());
            if (!Strings.isBlank(bucketFilter.getCannedAccessControlList()) && !objectAcl.getPermission().toString().substring(objectAcl.getPermission().toString().length()-3).equals(bucketFilter.getCannedAccessControlList().substring(bucketFilter.getCannedAccessControlList().length()-3))){
                oss.setObjectAcl(bucketFilter.getBucketName(), bucketFilter.getObjectName(), CannedAccessControlList.valueOf(bucketFilter.getCannedAccessControlList()));
            }
            oss.shutdown();
        }catch (Exception e){
            throw new BusinessException("出现错误,修改失败");
        }
        return true;
    }

    @Override
    public List<OssFileInfoDto> fileList(BucketFilter bucketFilter) {
        //TODO 前端处理时统一遇到"/"结尾 就显示为文件夹
        List<OssFileInfoDto> ossFileInfoDtoList = new ArrayList<>();
        try {
            OssClient ossClient = new OssClient().invoke(configurationService);
            OSS oss = new OSSClientBuilder().build(ossClient.getEndpoint(),ossClient.getAccessKeyId(),ossClient.getAccessKeySecret());
            // 构造ListObjectsRequest请求。
            ListObjectsRequest listObjectsRequest = new ListObjectsRequest(bucketFilter.getBucketName());
            // 设置prefix参数-->获取某个目录下的所有文件。
            if (!bucketFilter.getFolderName().isEmpty()) {
                listObjectsRequest.setPrefix(bucketFilter.getFolderName() + "/");
            }
            //设置正斜线("/")为文件夹的分隔符 commonPrefix->对应的值是当前文件夹下的子文件夹
            listObjectsRequest.setDelimiter("/");
            // 列出文件。
            ObjectListing listing = oss.listObjects(listObjectsRequest);

            // 遍历当前文件-->所有文件。
            for (OSSObjectSummary objectSummary : listing.getObjectSummaries()) {
                bucketFilter.setObjectName(objectSummary.getKey());
                ossFileInfoDtoList.add(fileInfo(bucketFilter));
            }

            // 遍历所有commonPrefix。-->列出当前文件夹下的子文件夹
            for (String commonPrefix : listing.getCommonPrefixes()) {
                ossFileInfoDtoList.add(OssFileInfoDto.builder().subFolderName(commonPrefix).build());
            }

            // 关闭OSSClient。
            oss.shutdown();
        }catch (Exception e){
            throw new BusinessException("发生异常,获取失败");
        }
        return ossFileInfoDtoList;
    }

    private void archiveToStandard(BucketFilter bucketFilter, OSS oss) throws InterruptedException {
        ObjectMetadata objectMetadata;
            // 解冻文件。
            oss.restoreObject(bucketFilter.getBucketName(), bucketFilter.getObjectName());

            // 等待解冻完成。
            do {
                Thread.sleep(1000);
                objectMetadata = oss.getObjectMetadata(bucketFilter.getBucketName(), bucketFilter.getObjectName());
            } while (!objectMetadata.isRestoreCompleted());

        // 创建CopyObjectRequest对象。
        CopyObjectRequest request = new CopyObjectRequest(bucketFilter.getBucketName(), bucketFilter.getObjectName(), bucketFilter.getBucketName(), bucketFilter.getObjectName()) ;

        // 创建ObjectMetadata对象。
        objectMetadata = new ObjectMetadata();

        // 封装header，此处以设置存储类型为低频访问类型为例。
        objectMetadata.setHeader("x-oss-storage-class", StorageClass.Standard);
        request.setNewObjectMetadata(objectMetadata);

        // 更改文件存储类型。
        oss.copyObject(request);
    }

    private void iAToArchive(BucketFilter bucketFilter, OSS oss) {
        // 创建CopyObjectRequest对象。
        CopyObjectRequest request = new CopyObjectRequest(bucketFilter.getBucketName(), bucketFilter.getObjectName(), bucketFilter.getBucketName(), bucketFilter.getObjectName()) ;
        // 创建ObjectMetadata对象。
        ObjectMetadata objectMetadata = new ObjectMetadata();

        // 封装header，设置存储类型为低频类型。
        objectMetadata.setHeader("x-oss-storage-class", StorageClass.Archive);
        request.setNewObjectMetadata(objectMetadata);

        // 更改文件存储类型。
        oss.copyObject(request);
    }

    private void iAToStandard(BucketFilter bucketFilter, OSS oss) {
        // 创建CopyObjectRequest对象。
        CopyObjectRequest request = new CopyObjectRequest(bucketFilter.getBucketName(), bucketFilter.getObjectName(), bucketFilter.getBucketName(), bucketFilter.getObjectName()) ;
        // 创建ObjectMetadata对象。
        ObjectMetadata objectMetadata = new ObjectMetadata();

        // 封装header，设置存储类型为低频类型。
        objectMetadata.setHeader("x-oss-storage-class", StorageClass.Standard);
        request.setNewObjectMetadata(objectMetadata);

        // 更改文件存储类型。
         oss.copyObject(request);
    }

    private void standardToIA(BucketFilter bucketFilter, OSS oss) {
        // 创建CopyObjectRequest对象。
        CopyObjectRequest request = new CopyObjectRequest(bucketFilter.getBucketName(), bucketFilter.getObjectName(), bucketFilter.getBucketName(), bucketFilter.getObjectName()) ;
        // 创建ObjectMetadata对象。
        ObjectMetadata objectMetadata = new ObjectMetadata();

        // 封装header，设置存储类型为低频类型。
        objectMetadata.setHeader("x-oss-storage-class", StorageClass.IA);
        request.setNewObjectMetadata(objectMetadata);

        // 更改文件存储类型。
        oss.copyObject(request);
    }

    private void archiveToIA(BucketFilter bucketFilter, OSS oss) throws InterruptedException {
        ObjectMetadata objectMetadata;
            // 解冻文件。
            oss.restoreObject(bucketFilter.getBucketName(), bucketFilter.getObjectName());

            // 等待解冻完成。
            do {
                Thread.sleep(1000);
                objectMetadata = oss.getObjectMetadata(bucketFilter.getBucketName(), bucketFilter.getObjectName());
            } while (!objectMetadata.isRestoreCompleted());

        // 创建CopyObjectRequest对象。
        CopyObjectRequest request = new CopyObjectRequest(bucketFilter.getBucketName(), bucketFilter.getObjectName(), bucketFilter.getBucketName(), bucketFilter.getObjectName()) ;

        // 创建ObjectMetadata对象。
        objectMetadata = new ObjectMetadata();

        // 封装header，此处以设置存储类型为低频访问类型为例。
        objectMetadata.setHeader("x-oss-storage-class", StorageClass.IA);
        request.setNewObjectMetadata(objectMetadata);

        // 更改文件存储类型。
        oss.copyObject(request);
    }

    private void standardToArchive(BucketFilter bucketFilter, OSS oss) {
        // 创建CopyObjectRequest对象。
        CopyObjectRequest request = new CopyObjectRequest(bucketFilter.getBucketName(), bucketFilter.getObjectName(), bucketFilter.getBucketName(), bucketFilter.getObjectName()) ;
        // 创建ObjectMetadata对象。
        ObjectMetadata objectMetadata = new ObjectMetadata();

        // 封装header，设置存储类型为归档类型。
        objectMetadata.setHeader("x-oss-storage-class", StorageClass.Archive);
        request.setNewObjectMetadata(objectMetadata);

        // 更改文件存储类型。
        oss.copyObject(request);
    }

    private List<String> selectNumberBucket(BucketFilter bucketFilter) {
        List<String> bucketList = new ArrayList<>();
        OssClient ossClient = new OssClient().invoke(configurationService);
        //创建实例
        OSS oss = new OSSClientBuilder().build(ossClient.getEndpoint(),ossClient.getAccessKeyId(),ossClient.getAccessKeySecret());
        ListBucketsRequest listBucketsRequest = new ListBucketsRequest();
        // 限定此次列举存储空间的个数为多少。默认值为100，最大值为1000。
        listBucketsRequest.setMaxKeys(Integer.parseInt(bucketFilter.getSelectBucketParam()));
        BucketList bucketL = oss.listBuckets(listBucketsRequest);
        for (Bucket bucket : bucketL.getBucketList()) {
            bucketList.add(bucket.getName());
        }
        return bucketList;
    }

    private List<String> selectMarkerBucket(BucketFilter bucketFilter) {
        List<String> bucketList = new ArrayList<>();
        OssClient ossClient = new OssClient().invoke(configurationService);
        //创建实例
        OSS oss = new OSSClientBuilder().build(ossClient.getEndpoint(),ossClient.getAccessKeyId(),ossClient.getAccessKeySecret());
        ListBucketsRequest listBucketsRequest = new ListBucketsRequest();
        // 列举指定前缀的存储空间。
        listBucketsRequest.setMarker(bucketFilter.getSelectBucketParam());
        BucketList bucketL = oss.listBuckets(listBucketsRequest);
        for (Bucket bucket : bucketL.getBucketList()) {
            bucketList.add(bucket.getName());
        }
        return bucketList;
    }

    private List<String> selectPrefixBucket(BucketFilter bucketFilter) {
        List<String> bucketList = new ArrayList<>();
        OssClient ossClient = new OssClient().invoke(configurationService);
        //创建实例
        OSS oss = new OSSClientBuilder().build(ossClient.getEndpoint(),ossClient.getAccessKeyId(),ossClient.getAccessKeySecret());
        ListBucketsRequest listBucketsRequest = new ListBucketsRequest();
        // 列举指定前缀的存储空间。
        listBucketsRequest.setPrefix(bucketFilter.getSelectBucketParam());
        BucketList bucketL = oss.listBuckets(listBucketsRequest);
        for (Bucket bucket : bucketL.getBucketList()) {
            bucketList.add(bucket.getName());
        }
        return bucketList;
    }

    private List<String> selectAllBucket() {
        List<String> bucketList = new ArrayList<>();
        OssClient ossClient = new OssClient().invoke(configurationService);
        //创建实例
        OSS oss = new OSSClientBuilder().build(ossClient.getEndpoint(),ossClient.getAccessKeyId(),ossClient.getAccessKeySecret());
        // 列举存储空间。
        List<Bucket> buckets = oss.listBuckets();
        for (Bucket bucket : buckets) {
            bucketList.add(bucket.getName());
        }
        oss.shutdown();
        return bucketList;
    }
}
