package cn.bs.bsmanagebasic.service.impl;

import cn.bs.bsmanagebasic.dao.jisu.MenuMapper;
import cn.bs.bsmanagebasic.entity.Menu;
import cn.bs.bsmanagebasic.entity.MenuRouter;
import cn.bs.bsmanagebasic.entity.MenuRouterMeta;
import cn.bs.bsmanagebasic.entity.User;
import cn.bs.bsmanagebasic.service.MenuService;
import cn.bs.bsmanagecommon.exception.BusinessException;
import cn.bs.bsmanagecommon.result.ResultBean;
import com.alibaba.fastjson.JSON;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

/**
 * @author LiuJS
 * @Time 2021/4/26 16:22
 * @Desc
 * @Version 1.0
 */
@Service
public class MenuServiceImpl implements MenuService {

    @Resource
    private MenuMapper menuMapper;

    @Override
    public List<Menu> list() {
        List<Menu> routerList = new ArrayList<>();
        List<Menu> menuList = menuMapper.list();
        // 将router部分分离
        for (Iterator<Menu> iterator = menuList.iterator(); iterator.hasNext();) {
            Menu menuIterator = iterator.next();
            Menu menu = new Menu();
            menu.setId(menuIterator.getId());
            menu.setParentId(menuIterator.getParentId());
            menu.setPath(menuIterator.getPath());
            menu.setMethod(menuIterator.getMethod());
            menu.setName(menuIterator.getName());
            menu.setDescription(menuIterator.getDescription());
            menu.setComponent(menuIterator.getComponent());
            menu.setTitle(menuIterator.getTitle());
            menu.setIcon(menuIterator.getIcon());
            menu.setType(menuIterator.getType());
            menu.setPermission(menuIterator.getPermission());

            routerList.add(menu);
            iterator.remove();
        }
        rebuildMenus(routerList);
        return routerList;
    }

    @Override
    public List<MenuRouter> getRoleRouter(HttpServletRequest request) {

        HttpSession session = request.getSession();
        User user = JSON.parseObject(session.getAttribute(session.getId()).toString(), User.class);
        List<Menu> menuList = menuMapper.getRoleRouter(user);
        List<MenuRouter> routerList = new ArrayList<>();
        // 将router部分分离
        for (Iterator<Menu> iterator = menuList.iterator(); iterator.hasNext();) {
            Menu menu = iterator.next();
            if (menu.getType().equals("menu")|| menu.getType().equals("router")) {

                MenuRouter resourceRouter = new MenuRouter();
                MenuRouterMeta meta = new MenuRouterMeta();

                resourceRouter.setId(menu.getId());
                resourceRouter.setParentId(menu.getParentId());
                resourceRouter.setPath(menu.getPath());
                resourceRouter.setName(menu.getName());
                resourceRouter.setType(menu.getType());
                resourceRouter.setComponent(menu.getComponent());
                meta.setTitle(menu.getTitle());
                meta.setIcon(menu.getIcon());
                resourceRouter.setMeta(meta);
                routerList.add(resourceRouter);
                iterator.remove();
            }
        }
        rebuildMenus2(routerList);
        return routerList;
    }

    @Override
    public List<String> getApiPermission(HttpServletRequest request) {
        HttpSession session = request.getSession();
        User user = JSON.parseObject(session.getAttribute(session.getId()).toString(), User.class);
        String roleId = menuMapper.getRoleIdByUserId(user.getUserId());
        if (roleId == null || "".equals(roleId)) {
            throw new BusinessException("请重新登录");
        }
        List<String> list = menuMapper.getApiPermission(roleId);
        return list;
    }

    @Override
    public boolean addMenu(Menu menu) {
        menu.setCreateTime(new Date());
        menuMapper.addResource(menu);
        return true;
    }

    @Override
    public boolean updateMenu(Menu menu) {
        menu.setCreateTime(new Date());
        menuMapper.updateResource(menu);
        return true;
    }

    @Override
    public boolean deleteMenu(Long id) {
        menuMapper.deleteResource(id);
        return true;
    }

    //处理菜单按钮
    private List<Menu> rebuildMenus(List<Menu> routerList) {
        List<Menu>  routerList2 = new ArrayList<>(routerList);

        List<Menu>  tempList ;
        List<Menu>  tempList2 = new ArrayList<>();
        for (Menu menuInfo: routerList){
            tempList = new ArrayList<>();
            for (Iterator<Menu> iterator = routerList2.iterator(); iterator.hasNext();){
                Menu menuInfo2 = iterator.next();
                if (menuInfo.equals(menuInfo2)){
                    continue;
                } else if( menuInfo.getId() == menuInfo2.getParentId()) {
                    tempList.add(menuInfo2);
                    tempList2.add(menuInfo2);
                    iterator.remove();
                }
            }
            if (tempList.size() != 0) {
                menuInfo.setChildren(tempList);
            }
        }
        routerList.removeAll(tempList2);
        if (tempList2.size() != 0) {
            rebuildMenus(routerList);
        }
        return routerList;
    }

    //处理菜单按钮
    private List<MenuRouter> rebuildMenus2(List<MenuRouter> routerList){
        List<MenuRouter>  routerList2 = new ArrayList<>(routerList);

        List<MenuRouter>  tempList ;
        List<MenuRouter>  tempList2 = new ArrayList<MenuRouter>();
        for (MenuRouter menuInfo: routerList){
            tempList = new ArrayList<MenuRouter>();
            for (Iterator<MenuRouter> iterator = routerList2.iterator(); iterator.hasNext();){
                MenuRouter menuInfo2 = iterator.next();
                if (menuInfo.equals(menuInfo2)){
                    continue;
                } else if( menuInfo.getId() == menuInfo2.getParentId()) {
                    tempList.add(menuInfo2);
                    tempList2.add(menuInfo2);
                    iterator.remove();
                }
            }
            if (tempList.size() != 0) {
                menuInfo.setChildren(tempList);
            }
        }
        routerList.removeAll(tempList2);
        if (tempList2.size() != 0) {
            rebuildMenus2(routerList);
        }
        return routerList;
    }


    @Override
    public List<Menu> listMenuAll(HttpServletRequest request) {
        List<Menu> routerList = new ArrayList<>();
        HttpSession session = request.getSession();
        User user = JSON.parseObject(session.getAttribute(session.getId()).toString(), User.class);
        List<Menu> menuList = menuMapper.listMenuAll(user.getRoleId());
        // 将router部分分离
        for (Iterator<Menu> iterator = menuList.iterator(); iterator.hasNext();) {
            Menu menuIterator = iterator.next();
            Menu menu = new Menu();
            menu.setId(menuIterator.getId());
            menu.setParentId(menuIterator.getParentId());
            menu.setPath(menuIterator.getPath());
            menu.setMethod(menuIterator.getMethod());
            menu.setName(menuIterator.getName());
            menu.setDescription(menuIterator.getDescription());
            menu.setComponent(menuIterator.getComponent());
            menu.setTitle(menuIterator.getTitle());
            menu.setIcon(menuIterator.getIcon());
            menu.setType(menuIterator.getType());
            menu.setPermission(menuIterator.getPermission());

            routerList.add(menu);
            iterator.remove();
        }
        rebuildMenus(routerList);
        return routerList;
    }
}
