package cn.bs.bsmanagebasic.service.impl;

import cn.bs.bsmanagebasic.config.RabbitConfig;
import cn.bs.bsmanagebasic.dao.jeesite.StoreImeiMapper;
import cn.bs.bsmanagebasic.dao.jeesite.TempNoticeMapper;
import cn.bs.bsmanagebasic.entity.TempNotice;
import cn.bs.bsmanagebasic.entity.filter.NoticeFilter;
import cn.bs.bsmanagebasic.entity.jiguang.ExtraBean;
import cn.bs.bsmanagebasic.entity.jiguang.JiguangBean;
import cn.bs.bsmanagebasic.service.NoticeService;
import cn.bs.bsmanagebasic.utils.MD5;
import cn.bs.bsmanagecommon.exception.BusinessException;
import com.google.gson.Gson;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

/**
 * 〈功能概述〉<br>
 *
 * @className: NoticeServiceImpl
 * @package: cn.bs.bsmanagebasic.service
 * @author: liujs
 * @date: 2019/11/27 0006 下午 15:43
 */
@Service
public class NoticeServiceImpl implements NoticeService {

    @Autowired
    private RabbitService rabbitService;

    @Autowired
    private RabbitConfig rabbitConfig;

    @Autowired
    private TempNoticeMapper tempNoticeMapper;

    @Resource(name = "storeImeiMapper")
    private StoreImeiMapper storeImeiMapper;

    @Override
    public Boolean sendNotice(NoticeFilter noticeFilter) {
        try{
            //获取需要发送的公告
            List<TempNotice> noticeList = tempNoticeMapper.getSendNotice(noticeFilter);
            if (noticeList == null || noticeList.size() == 0) {
                throw new BusinessException("没有查询到数据");
            }
            //对消息进行处理封装
            noticeList.forEach(notice -> {
                JiguangBean jiguangBean = new JiguangBean();
                ExtraBean extraBean = new ExtraBean();
                jiguangBean.setPushType(notice.getPushType());
                jiguangBean.setNotification_title(notice.getNoticeTitle());
                jiguangBean.setMsg_title(notice.getMsgTitle());
                jiguangBean.setMsg_content(notice.getMsgContent());
                jiguangBean.setPushRange(notice.getPushRange());

                extraBean.setType(notice.getExtraType());
                extraBean.setUrl(notice.getExtraUrl());
                extraBean.setIsPDF(notice.getExtraIsPDF());
                extraBean.setControllerIOS(notice.getExtraControllerIOS());
                extraBean.setControllerAndroid(notice.getExtraControllerAndroid());
                extraBean.setSound(notice.getExtraSound());
                extraBean.setParams(notice.getExtraParams());
                extraBean.setOperationType(notice.getExtraOperationType());
                jiguangBean.setExtraBean(extraBean);
                //根据别名推送
                Collection<String> alias = new HashSet<>();
                //推送多个门店时
                if (notice.getAliasObject() != null && !("").equals(notice.getAliasObject())) {
                    List<String> storeIdList = Arrays.asList(notice.getAliasObject().split(","));
                    //获取需要通知的设备号并且加密
                    storeIdList.forEach(storeId -> {
                        List<String> imeiList = getStoreImei(Integer.valueOf(storeId));
                        imeiList.forEach(imei -> alias.add(imei));
                        jiguangBean.setAlias(alias);
                    });
                }
                //将消息发送到队列中
                rabbitService.send(rabbitConfig.getJPushExchangeName(), rabbitConfig.getJPushRoutingKey(), new Gson().toJson(jiguangBean));
            });
        }catch (Exception e){
            e.printStackTrace();
        }
        return true;
    }

    private List<String> getStoreImei(Integer storeId) {
        List<String> imeiByJiaMiList = new ArrayList<>();
        List<String> imeiList= storeImeiMapper.getImeiByStoreId(storeId);
        imeiList.forEach(imei -> imeiByJiaMiList.add(MD5.md5(imei)));
        return imeiByJiaMiList;
    }

    @Override
    public Boolean addNotice(TempNotice tempNotice) {
        if(Strings.isBlank(tempNotice.getMsgContent())) {
            throw new BusinessException("参数错误");
        }
        return tempNoticeMapper.insert(tempNotice) > 0;
    }

    @Override
    public List<TempNotice> getList(NoticeFilter filter){
        filter.initSort();
        filter.initPage();
        return tempNoticeMapper.getList(filter);
    }

    @Override
    public Integer getListCount(NoticeFilter filter){
        return tempNoticeMapper.getListCount(filter);
    }

    @Override
    public boolean updateNotice(TempNotice notice){
        if(notice.getId() == null){
            throw new BusinessException("传入id不能为空");
        }
        return tempNoticeMapper.updateNotice(notice)>0;
    }

    @Override
    public boolean deleteNotice(List<Integer> ids){
        if(ids == null || ids.size() == 0){
            throw new BusinessException("传入参数不能为空");
        }
        return tempNoticeMapper.deleteNotice(ids)>0;
    }


}
