package cn.bs.bsmanagebasic.service.impl;

import cn.bs.bsmanagebasic.common.util.CacheTool;
import cn.bs.bsmanagebasic.dao.jeesite.PlatformMapper;
import cn.bs.bsmanagebasic.entity.Platform;
import cn.bs.bsmanagebasic.service.PlatformService;
import cn.bs.bsmanagecommon.exception.BusinessException;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author kael
 */
@Service
@Slf4j
public class PlatformServiceImpl implements PlatformService {

	@Autowired
	private PlatformMapper platformMapper;

	/**
	 * 查询平台信息
	 * @param platform
	 * @return
	 */
	@Override
	public List<Platform> getAll(){
		CacheTool cacheTool = CacheTool.initPlatformConfig();
		List<Platform> platforms = cacheTool.getBean("",Platform.class);
		if (platforms != null) {
			return platforms;
		} else {
			platforms = platformMapper.getAll();
			if (platforms != null) {
				platforms.stream().forEach(platform -> cacheTool.saveBean(platform.getPlatform(), platform));
				return platforms;
			}
			return null;
		}
	}

	@Override
	public boolean save(Platform platform){
		if(platform.getId() == null){
			return upRedisConfig(platform,platformMapper.insert(platform)>0);
		}else{
			return upRedisConfig(platform,platformMapper.update(platform)>0);
		}
	}

	@Override
	public boolean deleteById(Platform platform){
        if(platform.getId() == null){
            throw new BusinessException("传入id不能为空");
        }
        boolean fale = platformMapper.deleteById(platform.getId()) > 0;
        if(fale && !Strings.isBlank(platform.getPlatform())){
            CacheTool cacheTool = CacheTool.initPlatformConfig();
            cacheTool.del(platform.getPlatform());
        }
        return fale;
    }

	private boolean upRedisConfig(Platform platform, boolean fale) {
		if (fale) {
			CacheTool cacheTool = CacheTool.initPlatformConfig();
			cacheTool.saveBean(platform.getPlatform(), platform);
		}
		return fale;
	}
}
