package cn.bs.bsmanagebasic.service.impl;

import cn.bs.bsmanagebasic.dao.jeesite.SignBackSwitchMapper;
import cn.bs.bsmanagebasic.entity.SignBackSwitch;
import cn.bs.bsmanagebasic.entity.SignBackSwitchExample;
import cn.bs.bsmanagebasic.service.SignBackSwitchService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author sunjj
 * @version 1.0.0
 * @ClassName SignBackSwitchServiceImpl.java
 * @Description TODO
 * @createTime 2019年07月24日 15:19:00
 */
@Service
public class SignBackSwitchServiceImpl implements SignBackSwitchService {
    @Autowired
    private SignBackSwitchMapper mapper;

    @Override
    public int insert(SignBackSwitch signBackSwitch) {
        return mapper.insertSelective(signBackSwitch);
    }

    @Override
    public int insertList(List<SignBackSwitch> signBackSwitches) {
        return mapper.insertListByDefaultValue(signBackSwitches);
    }

    /**
     * @throws
     * @description 更新签收回传的状态
     * @author sunjj
     * @updateTime
     */
    @Override
    public boolean updateByStoreId(long storeId, List<SignBackSwitch> list) {
        SignBackSwitchExample example = new SignBackSwitchExample();
        SignBackSwitchExample.Criteria criteria = example.createCriteria();
        criteria.andStoreIdEqualTo(storeId);
        List<SignBackSwitch> oldSignBackSwitches = mapper.selectByExample(example);

        List<SignBackSwitch> insertSignBackSwitches = new ArrayList<>();
        boolean hasOld = null != oldSignBackSwitches && oldSignBackSwitches.size() > 0;
        boolean hasUpdate = null != list && list.size() > 0;

        if (hasOld && hasUpdate) {

            // 获取需要重新启用签收回传的快递公司
            List<Long> enabledExpIds = list.stream().filter(item ->
                    (oldSignBackSwitches.stream().anyMatch(match -> match.getExpId().equals(item.getExpId()) && !match.getIsValid()))
            ).map(item -> item.getExpId()).collect(Collectors.toList());
            if (null != enabledExpIds && enabledExpIds.size() > 0) {
                updateValid(storeId, enabledExpIds, true);
            }

            // 获取需要禁用签收回传的公司
            List<Long> disabledExpIds = oldSignBackSwitches.stream().filter(item ->
                    !(list.stream().anyMatch(match -> match.getExpId().equals(item.getExpId()))) && item.getIsValid()
            ).map(item -> item.getExpId()).collect(Collectors.toList());
            if (null != disabledExpIds && disabledExpIds.size() > 0) {
                updateValid(storeId, disabledExpIds, false);
            }

            // 获取本次新增开启签收回传的公司
            insertSignBackSwitches = list.stream().filter(item ->
                    !(oldSignBackSwitches.stream().anyMatch(match -> match.getExpId().equals(item.getExpId())))
            ).collect(Collectors.toList());

        } else if (hasOld && !hasUpdate) {
            updateValid(storeId, false);
        } else if (!hasOld && hasUpdate) {
            insertSignBackSwitches.addAll(list);
        }

        if (null != insertSignBackSwitches && insertSignBackSwitches.size() > 0) {
            for (SignBackSwitch backSwitch : insertSignBackSwitches) {
                insert(backSwitch);
            }
        }

        return true;
    }

    @Override
    public List<SignBackSwitch> selectByStoreId(long storeId, boolean isValid) {
        SignBackSwitchExample example = new SignBackSwitchExample();
        SignBackSwitchExample.Criteria criteria = example.createCriteria();
        criteria.andStoreIdEqualTo(storeId);
        criteria.andIsValidEqualTo(isValid);
        return mapper.selectByExample(example);
    }

    @Override
    public int batchUpdateByStoreId(SignBackSwitch signBackSwitch) {
        return mapper.batchUpdateByStoreId(signBackSwitch);
    }

    /**
     * @throws
     * @description 根据门店Id，更新启用状态
     */
    private int updateValid(long storeId, boolean valid) {
        SignBackSwitch updateBackSwitch = new SignBackSwitch();
        updateBackSwitch.setIsValid(valid);

        SignBackSwitchExample example1 = new SignBackSwitchExample();
        SignBackSwitchExample.Criteria criteria1 = example1.createCriteria();
        criteria1.andStoreIdEqualTo(storeId);

        return mapper.updateByExampleSelective(updateBackSwitch, example1);
    }

    /**
     * @throws
     * @description 根据门店Id，快递公司ID更新启用状态
     */
    private int updateValid(long storeId, List<Long> expIds, boolean valid) {
        SignBackSwitch updateBackSwitch = new SignBackSwitch();
        updateBackSwitch.setIsValid(valid);

        SignBackSwitchExample example = new SignBackSwitchExample();
        SignBackSwitchExample.Criteria criteria = example.createCriteria();
        criteria.andStoreIdEqualTo(storeId);
        criteria.andExpIdIn(expIds);

        return mapper.updateByExampleSelective(updateBackSwitch, example);
    }


}
