package cn.bs.bsmanagebasic.service.impl;

import cn.bs.bsmanagebasic.common.util.Pager;
import cn.bs.bsmanagebasic.dao.jisu.SmsRecordMapper;
import cn.bs.bsmanagebasic.entity.SmsRecord;
import cn.bs.bsmanagebasic.entity.TempNotice;
import cn.bs.bsmanagebasic.entity.User;
import cn.bs.bsmanagebasic.entity.filter.NoticeFilter;
import cn.bs.bsmanagebasic.entity.filter.SmsRecordFilter;
import cn.bs.bsmanagebasic.service.ConfigurationService;
import cn.bs.bsmanagebasic.service.NoticeService;
import cn.bs.bsmanagebasic.service.SmsService;
import cn.bs.bsmanagebasic.utils.OkHttpUtil;
import cn.bs.bsmanagecommon.exception.BusinessException;
import com.alibaba.fastjson.JSON;
import com.google.common.base.Strings;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 发送短信<br>
 *
 * @className: SmsServiceImpl
 * @package: cn.bs.bsmanagebasic.service.impl
 * @author: kael
 * @date: 2019/10/16 0016 下午 17:13
 */
@Service
@Slf4j
public class SmsServiceImpl implements SmsService {

    @Autowired
    private SmsRecordMapper smsRecordMapper;

    @Autowired
    private OkHttpUtil okHttpUtil;

    @Autowired
    private ConfigurationService configurationService;

    @Autowired
    private NoticeService noticeService;

    @Override
    public boolean resendSms(final SmsRecordFilter filter){
        try {
//            filter.setSendSuccessType(1);
            List<SmsRecord> smsRecords = getSmsRecordList(filter);
            Map<String,Object> param = new HashMap<>(16);
            String url = configurationService.getConfigurationByCode("smsMsgUrl").getOValue1();
            smsRecords.stream().collect(Collectors.groupingBy(SmsRecord::getStoreId)).entrySet().stream().forEach(entry -> {
                param.put("storeId",entry.getKey().toString());
                param.put("ids", entry.getValue().stream().map(SmsRecord::getDeliveryId).collect(Collectors.toList()));
//                param.put("platform",filter.getPlatform());
                String res = okHttpUtil.doPostJson(url+"/sms/resendSms",JSON.toJSONString(param));
                log.info("【门店{}重发短信接口返回参数】{}",entry.getKey().toString(),res);
            });
            return true;
        }catch (Exception e){
            e.printStackTrace();
            return false;
        }
    }

    @Override
    public List<SmsRecord> getSmsRecordList(SmsRecordFilter filter){
        return smsRecordMapper.getSmsRecordList(filter);
    }

    @Override
    public Pager<SmsRecord> getList(SmsRecordFilter filter, HttpServletRequest request){
        HttpSession session = request.getSession();
        User user = JSON.parseObject(session.getAttribute(session.getId()).toString(), User.class);
        filter.setStoreId(0 == user.getStoreId().intValue() ? null : user.getStoreId().intValue());
        filter.initSort();
        filter.initPage();
        Pager<SmsRecord> pager = new Pager<SmsRecord>(getCount(filter), filter.getPageNumber(), filter.getPageSize());
        pager.setList(getSmsRecordList(filter));
        return pager;
    }

    @Override
    public Integer getCount(SmsRecordFilter filter){
        filter.initSort();
        filter.initPage();
        return smsRecordMapper.getSmsRecordCount(filter);
    }

    @Override
    public Pager<TempNotice> getNotice(NoticeFilter filter){
        filter.initPage();
        Pager<TempNotice> pager = new Pager<TempNotice>(noticeService.getListCount(filter), filter.getPageNumber(), filter.getPageSize());
        pager.setList(noticeService.getList(filter));
        return pager;
    }

    @Override
    public boolean saveNotice(TempNotice notice){
        if(notice.getId() == null){
            return noticeService.addNotice(notice);
        }else{
            return noticeService.updateNotice(notice);
        }
    }

    @Override
    public boolean deleteNotice(List<Integer> ids){
        return noticeService.deleteNotice(ids);
    }
}
