package cn.bs.bsmanagebasic.service.impl;

import cn.bs.bsmanagebasic.dao.jeesite.ExpDataStatisticalMapper;
import cn.bs.bsmanagebasic.entity.dto.ExpDataStatisticalDto;
import cn.bs.bsmanagebasic.entity.filter.StatisticsFilter;
import cn.bs.bsmanagebasic.service.StatisticsService;
import cn.bs.bsmanagecommon.result.ResultBean;
import cn.bs.bsmanagecommon.util.DateUtil;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.util.*;

/**
 * @author sunjj
 * @version 1.0.0
 * @ClassName StatisticsServiceImpl.java
 * @Description TODO
 * @createTime 2019年11月12日 16:01:00
 */
@Service
public class StatisticsServiceImpl implements StatisticsService {

    @Autowired
    private ExpDataStatisticalMapper mapper;

    @Override
    public ResultBean<Object> getHistogram(StatisticsFilter filter) {
        String dataKind = filter.getDataKind();
        if (Strings.isBlank(dataKind)) {
            // 默认统计当日数据
            filter.setTimeUnit("D");
            filter.setTimePeriod(1);
            getFilterDate(filter);
            ExpDataStatisticalDto dto = mapper.sumByFilter(filter);
            if (null == dto) {
                dto = new ExpDataStatisticalDto();
            }

            dto.setSmsNum((null == dto.getSmsFailNum() ? 0 : dto.getSmsFailNum()) + (null == dto.getSmsSuccessNum() ? 0 : dto.getSmsSuccessNum()));
            dto.setSignBackNum((null == dto.getSignBackSuccessNum() ? 0 : dto.getSignBackSuccessNum()) + (null == dto.getSignBackfailNum() ? 0 : dto.getSignBackfailNum()));
            if (null == dto.getOutbountNum() || dto.getOutbountNum().compareTo(0) == 0) {
                dto.setOutbountRate("0.00");
            } else if (null == dto.getInboundNum() || dto.getInboundNum().compareTo(0) == 0) {
                dto.setOutbountRate(dto.getOutbountNum() + ".00");
            } else {
                dto.setOutbountRate(String.format("%.2f", ((double) dto.getOutbountNum() / dto.getInboundNum() * 100)));
            }

            dto.setSevenOutbountRate(mapper.selectSevendayOutbountRate().getOutbountRate());

            ExpDataStatisticalDto onlineDto = mapper.selectOnline();
            dto.setStoreOnlineNum(onlineDto.getStoreOnlineNum());
            dto.setStoreSumNum(onlineDto.getStoreSumNum());

            return new ResultBean<>(dto);
        }

        getFilterDate(filter);

        List<ExpDataStatisticalDto> list;
        if ("D".equals(filter.getTimeUnit().toUpperCase()) && filter.getTimePeriod().compareTo(1) == 0) {
            list = mapper.selectByFilterEachHour(filter);
        } else {
            list = mapper.selectByFilterEachDay(filter);
        }

        return new ResultBean<>(getHistogramData(list, filter.getDataKind()));
    }

    @Override
    public ResultBean getPieChart(StatisticsFilter filter) {
        getFilterDate(filter);
        ExpDataStatisticalDto expDataStatisticalDto = mapper.sumByFilter(filter);
        return new ResultBean(expDataStatisticalDto);
    }

    private void getFilterDate(StatisticsFilter filter) {
        if (Strings.isBlank(filter.getTimeUnit())) {
            filter.setTimeUnit("D");
        }

        if (null == filter.getTimePeriod()) {
            filter.setTimePeriod(1);
        }

        if ("D".equals(filter.getTimeUnit().toUpperCase())) {
            // 获取当日数据
            if (filter.getTimePeriod().compareTo(1) == 0) {
                // 每日凌晨01：00前，统计昨日的数据
                // 例如2019-11-14 00：59：59 统计2019-11-13的数据
                Date beginTime = DateUtil.addHour(-1);
                filter.setBeginTime(DateUtil.formatDate(beginTime));
                try {
                    filter.setEndTime(DateUtil.addDay(filter.getBeginTime(), DateUtil.DATE_PATTERN, 1));
                } catch (ParseException e) {
                }
            } else {
                filter.setBeginTime(DateUtil.addDay(-filter.getTimePeriod().intValue(), "yyyy-MM-dd"));
            }
        } else {
            filter.setBeginTime(DateUtil.addMonth(-filter.getTimePeriod().intValue(), "yyyy-MM-dd"));
        }
    }

    private Map<String, Object> getHistogramData(List<ExpDataStatisticalDto> list, String dataKind) {
        Map<String, Object> resultMap = new HashMap<>();
        if (null != list && list.size() > 0) {
            List<String> date = new ArrayList<>();
            List<Object> data = new ArrayList<>();
            list.stream().forEach(item -> {
                date.add(item.getStatisticalTimeStr());
                switch (dataKind.toUpperCase()) {
                    case "IN":
                        data.add(item.getInboundNum());
                        break;
                    case "OUT":
                        data.add(item.getOutbountNum());
                        break;
                    case "OUT_RATE":
                        if (null == item.getOutbountNum() || item.getOutbountNum().compareTo(0) == 0) {
                            data.add("0.00");
                        } else if (null == item.getInboundNum() || item.getInboundNum().compareTo(0) == 0) {
                            data.add(item.getOutbountNum() + ".00");
                        } else {
                            data.add(String.format("%.2f", ((double) item.getOutbountNum() / item.getInboundNum() * 100)));
                        }
                        break;
                    case "SMS":
                        data.add((null == item.getSmsSuccessNum() ? 0 : item.getSmsSuccessNum()) + (null == item.getSmsFailNum() ? 0 : item.getSmsFailNum()));
                        break;
                    default:
                        break;
                }
            });
            resultMap.put("date", date);
            resultMap.put("data", data);
        }
        return resultMap;
    }
}
