package cn.bs.bsmanagebasic.service.impl;

import cn.bs.bsmanagebasic.common.constants.Constants.AppConfigConstants;
import cn.bs.bsmanagebasic.common.constants.Constants.ConsumeRecordConstants;
import cn.bs.bsmanagebasic.common.constants.Constants.SmspurchaserecordConstants;
import cn.bs.bsmanagebasic.common.enums.AdminTypeEnum;
import cn.bs.bsmanagebasic.common.util.CacheTool;
import cn.bs.bsmanagebasic.common.util.Pager;
import cn.bs.bsmanagebasic.dao.jeesite.ExStoreMapper;
import cn.bs.bsmanagebasic.dao.jisu.*;
import cn.bs.bsmanagebasic.entity.*;
import cn.bs.bsmanagebasic.entity.amap.GeoCodeResponse;
import cn.bs.bsmanagebasic.entity.dto.AdminUserListDto;
import cn.bs.bsmanagebasic.entity.dto.StoreDto;
import cn.bs.bsmanagebasic.entity.user.AdminUser;
import cn.bs.bsmanagebasic.entity.user.UserRole;
import cn.bs.bsmanagebasic.service.*;
import cn.bs.bsmanagebasic.utils.StringsUtil;
import cn.bs.bsmanagecommon.exception.BusinessException;
import com.alibaba.fastjson.JSON;
import com.google.common.base.Strings;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service
public class StoreManageServiceImpl implements StoreManageService {

    @Autowired
    private ExStoreMapper exStoreMapper;
    @Autowired
    private FjUserMapper fjUserMapper;
    @Autowired
    private StoreShelfMapper storeShelfMapper;
    @Autowired
    private UserService userService;
    @Autowired
    private StoreDeliveryMsgTemplateConfigMapper storeDeliveryMsgTemplateConfigMapper;
    @Autowired
    @Qualifier("jisuJdbcTemplate")
    private JdbcTemplate jdbcTemplate;

    @Autowired
    private AddressProvinceService addressProvinceService;
    @Autowired
    private AddressCityService addressCityService;
    @Autowired
    private AddressDistrictService addressDistrictService;
    @Autowired
    private SignBackSwitchService signBackSwitchService;
    @Autowired
    private StoreTemplateService storeTemplateService;
    @Autowired
    private ConsumeRecordService consumeRecordService;
    @Autowired
    private SmspurchaserecordService smspurchaserecordService;
    @Autowired
    private AppConfigService appConfigService;

    @Autowired
    private IAmapService amapService;

    @Resource
    private RoleMapper roleMapper;

    @Autowired
    private UserMapper userMapper;
    /**
     * @throws
     * @description 创建一个新的门店
     * @author sunjj
     * @updateTime
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean addStore(String storeData) {
        StoreDto storeDto = JSON.parseObject(storeData, StoreDto.class);
        // TODO 非空校验

        // 处理电话特殊字符问题
        dealPhoneNumber(storeDto);
        // 判断重复注册,门店账号重复(userid)
        User checkUserId = userService.getByPrimaryKey(storeDto.getUserId());
        if (checkUserId != null) {
            throw new BusinessException("用户已注册");
        }

        ExStore exStore = new ExStore();
        User user = new User();
        BeanUtils.copyProperties(storeDto, exStore);
        BeanUtils.copyProperties(storeDto, user);

        // 插入门店数据
        // 补充省市区信息
        List<AddressProvince> provinces = addressProvinceService.getByProvinceId(storeDto.getProvinceId().intValue());
        if (null != provinces && provinces.size() > 0) {
            exStore.setProvinceName(provinces.get(0).getProvinceName());
            exStore.setProvinceId(Integer.valueOf(String.valueOf(storeDto.getProvinceId()).substring(0,2)+"0000"));
        }
        List<AddressCity> cities = addressCityService.getByCityId(storeDto.getCityId());
        if (null != cities && cities.size() > 0) {
            exStore.setCityName(cities.get(0).getCityName());
            exStore.setCityId(Integer.valueOf(String.valueOf(storeDto.getCityId()).substring(0,4)+"00"));
        }
        List<AddressDistrict> districts = addressDistrictService.getByDistrictId(storeDto.getAreaId());
        if (null != districts && districts.size() > 0) {
            exStore.setAreaName(districts.get(0).getDistrictName());
            exStore.setAreaId(Integer.valueOf(String.valueOf(storeDto.getAreaId()).substring(0,6)));
        }
        exStore.setRegionCode(storeDto.getProvinceId() + "," + storeDto.getCityId() + "," + storeDto.getAreaId());
        // 补充备用手机号
        exStore.setBackupPhone(exStore.getPhone());

        initDefaultValue(exStore);
        initGeoCode(exStore);
        if (Strings.isNullOrEmpty(exStore.getShortName())) {
            exStore.setShortName(exStore.getName());
        }
        exStoreMapper.insertSelective(exStore);
        Long storeId = exStore.getId();

        // 插入门店对应user数据
        user.setStoreId(storeId);
        user.setUserName(exStore.getName());
        user.setTelphone(exStore.getPhone());
        initDefaultValue(user);
        userService.save(user);

        // 插入门店对应fj_user数据
        FjUser fjUser = new FjUser();
        fjUser.setStoreIds(storeId + "");
        initDefaultValue(fjUser);
        fjUserMapper.insertSelective(fjUser);

        // 插入门店对应货架数据
        List<StoreShelf> storeShelf = getStoreShelf(storeDto, storeId);
        storeShelfMapper.insertList(storeShelf);

        // 短信模板配置
        StoreDeliveryMsgTemplateConfig template = new StoreDeliveryMsgTemplateConfig();
        template.setStoreId(storeId.intValue());
        template.setStoreAddress(storeDto.getAddress());
        template.setCreateTime(new Date());
        initDefaultValue(template, 1);
        storeDeliveryMsgTemplateConfigMapper.insertSelective(template);
        initDefaultValue(template, 2);
        storeDeliveryMsgTemplateConfigMapper.insertSelective(template);

        // 短信模板配置
        StoreTemplate storeTemplate = new StoreTemplate();
        storeTemplate.setStoreId(storeId);
        storeTemplate.setTemplateId(storeDto.getTemplateId());
        storeTemplateService.saveByDefaultTemplate(storeTemplate);

        // 签收回传配置
        List<SignBackSwitch> signBackSwitches = new ArrayList<>();
        List<Integer> list = storeDto.getSignBackSwitch();
        for (Integer i : list) {
            SignBackSwitch signBackSwitch = new SignBackSwitch();
            signBackSwitch.setStoreId(storeId);
            signBackSwitch.setIsValid(true);
            signBackSwitch.setExpId(i.longValue());
            signBackSwitches.add(signBackSwitch);
        }
        if (null != list && list.size() > 0) {
            signBackSwitchService.insertList(signBackSwitches);
        }

        // app菜单配置
        RConfigGruop configGruop = new RConfigGruop();
        configGruop.setGroupKey(AppConfigConstants.GROUP_KEY_DEFAULT);
        configGruop.setStoreId(storeId);
        appConfigService.saveRConfigGruop(configGruop);

        // 创建门店对应业务表
        createDeliveryTable(storeId);
        return true;
    }

    /**
     * 处理电话特殊字符问题
     */
    private void dealPhoneNumber(StoreDto storeDto) {
        String userId;
        if (StringUtils.isNotEmpty(userId = storeDto.getUserId())) {
            userId = StringsUtil.removeNonAscii(userId);
            userId = StringsUtil.removeSomeControlChar(userId);
            userId = StringsUtil.removeFullControlChar(userId).trim();
            storeDto.setUserId(userId);
        }

        String phone;
        if(StringUtils.isNotEmpty(phone = storeDto.getPhone())){
            phone = StringsUtil.removeNonAscii(phone);
            phone = StringsUtil.removeSomeControlChar(phone);
            phone = StringsUtil.removeFullControlChar(phone).trim();
            storeDto.setPhone(phone);
        }

        String takeExpressPhone;
        if(StringUtils.isNotEmpty(takeExpressPhone = storeDto.getTakeExpressPhone())){
            takeExpressPhone = StringsUtil.removeNonAscii(takeExpressPhone);
            takeExpressPhone = StringsUtil.removeSomeControlChar(takeExpressPhone);
            takeExpressPhone = StringsUtil.removeFullControlChar(takeExpressPhone).trim();
            storeDto.setTakeExpressPhone(takeExpressPhone);
        }
    }

    private void initGeoCode(ExStore exStore) {
        try {
            String address = new StringBuilder(exStore.getProvinceName()).append(exStore.getCityName()).append(exStore.getAreaName()).append(exStore.getAddress()).toString();
            GeoCodeResponse geocode = amapService.geocode(address, exStore.getCityName());
            String location;
            if (geocode != null && geocode.getGeocodes() != null && geocode.getGeocodes().size() > 0 && !Strings.isNullOrEmpty(location = geocode.getGeocodes().get(0).getLocation())) {
                String[] split = location.split(",");
                exStore.setLongitude(split[0]);
                exStore.setLatitude(split[1]);
            }
        } catch (Exception e) {

        }
    }

    @Override
    public Pager<StoreDto> getListByParam(StoreDto dto, HttpServletRequest request) {
        HttpSession session = request.getSession();
        User user = JSON.parseObject(session.getAttribute(session.getId()).toString(), User.class);
        dto.setId(user.getStoreId() == 0 ? null : user.getStoreId());
        if (null == dto) {
            dto = new StoreDto();
        }
        // 初始化省市区查询条件
        initSelectParam(dto);
        long totalCount = exStoreMapper.countByParam(dto);
        if (dto.getPageSize() == 0) {
            dto.setPageSize(10);
        }
        if (dto.getPageNumber() == 0) {
            dto.setPageNumber(1);
        }

        Pager<StoreDto> pager = new Pager<StoreDto>(totalCount, dto.getPageNumber(), dto.getPageSize());

        dto.setBegin((pager.getPageNumber() - 1) * pager.getpageSize());
        pager.setList(exStoreMapper.selectDtoByParam(dto));
        return pager;
    }

    @Override
    public List<ExStore> getListStore() {
        List<ExStore> stores;
        stores = exStoreMapper.selectByExample(new ExStoreExample());
        return stores;
    }

    @Override
    public boolean editInfo(StoreDto storeDto) {
        // 判断重复注册,门店账号重复(userid)
        User checkUser = userService.getByPrimaryKey(storeDto.getUserId());
        if (checkUser != null && checkUser.getStoreId() != storeDto.getId().intValue()) {
            throw new BusinessException("用户已注册");
        }
        List<User> check = userService.getByStoreId(storeDto.getId());
        if (null != check && check.size() > 0 && !check.get(0).getUserId().equals(storeDto.getUserId())) {
            throw new BusinessException("请不要随意修改门店账号");
        }

        ExStore exStore = new ExStore();
        User user = new User();
        BeanUtils.copyProperties(storeDto, exStore);
        BeanUtils.copyProperties(storeDto, user);
        Long storeId = storeDto.getId();

        // 更新门店信息
        // 补充省市区信息
        List<AddressProvince> provinces = addressProvinceService.getByProvinceId(storeDto.getProvinceId().intValue());
        exStore.setProvinceId(null);
        if (null != provinces && provinces.size() > 0) {
            exStore.setProvinceName(provinces.get(0).getProvinceName());
        }
        List<AddressCity> cities = addressCityService.getByCityId(storeDto.getCityId());
        exStore.setCityId(null);
        if (null != cities && cities.size() > 0) {
            exStore.setCityName(cities.get(0).getCityName());
        }
        List<AddressDistrict> districts = addressDistrictService.getByDistrictId(storeDto.getAreaId());
        exStore.setAreaId(null);
        if (null != districts && districts.size() > 0) {
            exStore.setAreaName(districts.get(0).getDistrictName());
        }
        exStore.setRegionCode(storeDto.getProvinceId() + "," + storeDto.getCityId() + "," + storeDto.getAreaId());
        exStoreMapper.updateByPrimaryKeySelective(exStore);
        // 更新Redis
        updateRedis(storeId);

        // 更新User信息
        user.setUserId(null);
        user.setStoreId(exStore.getId());
        userService.updateByStoreId(user);

        // 更新短信模板信息
        StoreTemplate storeTemplate = new StoreTemplate();
        storeTemplate.setStoreId(storeId);
        storeTemplate.setTemplateId(storeDto.getTemplateId());
        storeTemplateService.updateTemplateByStoreId(storeId, storeDto.getTemplateId());

        // 更新短信模板配置
        StoreDeliveryMsgTemplateConfig templateConfig = new StoreDeliveryMsgTemplateConfig();
        templateConfig.setStoreAddress(exStore.getAddress());
        StoreDeliveryMsgTemplateConfigExample example = new StoreDeliveryMsgTemplateConfigExample();
        StoreDeliveryMsgTemplateConfigExample.Criteria criteria = example.createCriteria();
        criteria.andStoreIdEqualTo(storeId.intValue());
        storeDeliveryMsgTemplateConfigMapper.updateByExampleSelective(templateConfig,example);

        // 更新签收回传信息
        List<Integer> list = storeDto.getSignBackSwitch();
        List<SignBackSwitch> backSwitches = new ArrayList<SignBackSwitch>();
        for (Integer i : list) {
            SignBackSwitch signBackSwitch = new SignBackSwitch();
            signBackSwitch.setExpId(i.longValue());
            signBackSwitch.setStoreId(storeId);
            signBackSwitch.setIsValid(true);
            backSwitches.add(signBackSwitch);
        }
        signBackSwitchService.updateByStoreId(storeId, backSwitches);
        // 签收回传同步门店信息控制
        if (null != storeDto.getUpExpPhone() && storeDto.getUpExpPhone().booleanValue()) {
            SignBackSwitch signBackSwitch = new SignBackSwitch();
            // 需要重新同步数据
            signBackSwitch.setIsSyUpStore(false);
            signBackSwitch.setStoreId(storeId);
            signBackSwitchService.batchUpdateByStoreId(signBackSwitch);
        }
        return true;
    }

    /**
     * @throws
     * @description 充值
     * @author sunjj
     * @updateTime
     */
    @Override
    public Boolean money(Long id, BigDecimal money) {
        List<User> users = userService.getByStoreId(id);
        int status = 0;
        if (null != users && users.size() > 0) {
            User user = users.get(0);
            BigDecimal preBalance = user.getSmsBalance();
            BigDecimal aftBalance = preBalance.add(money);
            try {
                ConsumeRecord consumeRecord = new ConsumeRecord();
                consumeRecord.setConsumeFlag(ConsumeRecordConstants.CONSUME_FLAG_RECHARGE);
                consumeRecord.setStoreId(user.getStoreId());
                consumeRecord.setConsumeStatus(ConsumeRecordConstants.CONSUME_STATUS_SUCESS);
                consumeRecord.setOperatorId(BigDecimal.valueOf(Long.valueOf(user.getUserId())));
                consumeRecord.setAftBalance(aftBalance);
                consumeRecord.setPreBalance(preBalance);
                consumeRecord.setNowBalance(money);
                consumeRecordService.save(consumeRecord);
            } catch (Exception e) {
                throw new BusinessException("消费记录保存失败");
            }

            try {
                Smspurchaserecord record = new Smspurchaserecord();
                record.setAmount(money);
                record.setOperator(user.getUserId());
                // 已支付
                record.setPayStatus(SmspurchaserecordConstants.PAY_STATUS_PAID);
                // 线下充值
                record.setPayWay(SmspurchaserecordConstants.PAY_WAY_OFFLINE);
                // TODO： 猜测为充值的短信条数
                if (null != user.getSmsPrice()) {
                    record.setSmsNumber((int) (money.doubleValue() / user.getSmsPrice().doubleValue()));
                } else {
                    record.setSmsNumber(1);
                }
                record.setStoreId(user.getStoreId());
                smspurchaserecordService.save(record);
            } catch (Exception e) {
                throw new BusinessException("账单保存失败");
            }

            user.setSmsBalance(aftBalance);
            status = userService.updateByStoreId(user);

            CacheTool cacheTool = CacheTool.initUser();
            User user2 = userService.getByPrimaryKey(user.getUserId());
            cacheTool.saveBean(user.getUserId(), user2);
        }
        return status > 0;
    }

    @Override
    public Boolean disable(StoreDto dto) {
        ExStore store = new ExStore();
        store.setStatus(0);
        store.setId(dto.getId());
        int status = exStoreMapper.updateByPrimaryKeySelective(store);

        User user = new User();
        user.setStatus(0);
        user.setStoreId(dto.getId());
        userService.updateByStoreId(user);
        return status > 0;
    }

    @Override
    public List<Integer> selectIdByRegionCode(String area) {
        return exStoreMapper.selectIdByRegionCode(area);
    }

    @Override
    public ExStore getStoreFromCacheById(Long storeId) {
        CacheTool cacheTool = CacheTool.initStore();
        ExStore store = (ExStore) cacheTool.getBean(String.valueOf(storeId));
        if (null == store) {
            store = exStoreMapper.selectByPrimaryKey(storeId);
            if (null != store) {
                cacheTool.saveBean(String.valueOf(store.getId()), store);
            }
        }
        return store;
    }

    @Override
    public StoreDto getStoreById(StoreDto dto) {
        if (dto == null || dto.getId() == null) {
            throw new BusinessException("传入参数不能为空");
        }
        //目前直接查询数据库，后期需要去缓存中查询门店
        ExStore exStore = exStoreMapper.selectStoreById(dto.getId());
        if (exStore == null || exStore.getId() == null) {
            throw new BusinessException("门店id为：" + dto.getId() + "的门店不存在");
        }
        StoreDto storeDto = new StoreDto();
        BeanUtils.copyProperties(exStore, storeDto);
        List<SignBackSwitch> signBackSwitches = signBackSwitchService.selectByStoreId(storeDto.getId(), true);
        List<Integer> signInt = new ArrayList<Integer>();
        for (SignBackSwitch signBackSwitche : signBackSwitches) {
            signInt.add(signBackSwitche.getExpId().intValue());
        }
        storeDto.setSignBackSwitch(signInt);
        UserExample userExample = new UserExample();
        UserExample.Criteria userCriteria = userExample.createCriteria();
        userCriteria.andStoreIdEqualTo(storeDto.getId());
        List<User> userList = userService.selectByExample(userExample);
        if (userList == null || userList.size() == 0) {
            throw new BusinessException("用户不存在");
        }
        storeDto.setUserId(userList.get(0).getUserId());
        storeDto.setSmsPrice(userList.get(0).getSmsPrice());
        storeDto.setReminder(userList.get(0).getReminder());
        storeDto.setSmsBalance(userList.get(0).getSmsBalance());
        storeDto.setComSign(userList.get(0).getComSign());
        storeDto.setSmsSendType(userList.get(0).getSmsSendType());
        storeDto.setUsermail(userList.get(0).getUsermail());
        storeDto.setSiduanma(userList.get(0).getSiduanma());
        storeDto.setStatus(userList.get(0).getStatus().intValue());
        return storeDto;
    }

    /**
     * TODO 数据库持久化默认信息
     */
    private void initDefaultValue(ExStore exStore) {
        exStore.setCollegeId(1);
        exStore.setReceiveIds("202,187,272,17,18,223,243,244,273,274,275,276,277,279,280,282,285,287,288,289,290,291,292,19,293,294");
        exStore.setDispatchIds("202,187,272,17,18,223,243,244,273,274,275,276,277,279,280,282,285,287,288,289,290,291,292,19,293,294,999");
        exStore.setIsP2p(0);
        exStore.setIsPrinter(1);
        exStore.setLongitude("1");
    }

    /**
     * TODO 数据库持久化默认信息
     */
    private void initDefaultValue(FjUser fjUser) {
        fjUser.setUserId("fj" + fjUser.getStoreIds());
        fjUser.setPassword("e10adc3949ba59abbe56e057f20f883e");
    }

    /**
     * TODO 数据库持久化默认信息
     */
    private void initDefaultValue(User user) {
        user.setPassword("e10adc3949ba59abbe56e057f20f883e");
        user.setPush10(0);
        user.setPush15(0);
        user.setCodeType(0);
        user.setWebVersion("2.0");
        user.setContainsShelf(1);
        user.setComSign(user.getComSign());
        user.setAdminType(AdminTypeEnum.STORE_ADMIN.getType());
        initDefaultRole(user);
//        user.setRoleId("d7fe9d0a302f46ffa6210f3057e88efd");
    }

    /**
     * 门店创建默认角色
     * @param user
     */
    private void initDefaultRole(User user) {
        UserRole userRole = new UserRole();
        userRole.setUserId(user.getUserId());
        userRole.setRoleId(roleMapper.getStoreRoleDefault(1));
        userRole.setStoreId(user.getStoreId());
        userRole.setAdminType(AdminTypeEnum.STORE_ADMIN.getType());
        userMapper.addUserRole(userRole);
    }
    /**
     * TODO 数据库持久化默认信息
     */
    private void initDefaultValue(StoreDeliveryMsgTemplateConfig template, int flag) {
        if (flag == 1) {
            template.setClientType(6);
        } else {
            template.setClientType(7);
        }
        template.setType(1);
        template.setContainsTakeCode(1);
        template.setContainsShelf(1);

    }

    /**
     * TODO 数据库持久化默认信息
     */
    private List<StoreShelf> getStoreShelf(StoreDto storeDto, Long storeId) {
        List<StoreShelf> list = new ArrayList<>();
        int count = 1;
        for (int i = 1; i <= storeDto.getShelfCount(); i++) {
            for (int j = 1; j <= storeDto.getShelfLayers(); j++) {
                StoreShelf storeShelf = new StoreShelf();
                storeShelf.setStoreId(storeId.intValue());
                // storeShelf.setShelfTelephone(storeDto.getPhone());
                storeShelf.setShelfName(String.format("%s-%s", i, j));
                storeShelf.setBarCode(String.format("HJ %s%s", storeShelf.getStoreId(), String.format("%04d", count)));
                storeShelf.setIsdel(0);
                if (i == 1 && j == 1) {
                    storeShelf.setIsDefault(1);
                }else {
                    storeShelf.setIsDefault(0);
                }
                list.add(storeShelf);
                count++;
            }
        }
        return list;
    }

    private void initSelectParam(StoreDto dto) {
        if (null != dto.getProvinceId() && dto.getProvinceId() != 0) {
            List<AddressProvince> provinces = addressProvinceService.getByProvinceId(dto.getProvinceId().intValue());
            if (null != provinces && provinces.size() > 0) {
                dto.setProvinceName(provinces.get(0).getProvinceName());
            }
        }
        if (null != dto.getCityId() && dto.getCityId() != 0) {
            List<AddressCity> cities = addressCityService.getByCityId(dto.getCityId());
            if (null != cities && cities.size() > 0) {
                dto.setCityName(cities.get(0).getCityName());
            }
        }
        if (null != dto.getAreaId() && dto.getAreaId() != 0) {
            List<AddressDistrict> districts = addressDistrictService.getByDistrictId(dto.getAreaId());
            if (null != districts && districts.size() > 0) {
                dto.setAreaName(districts.get(0).getDistrictName());
            }
        }
    }

    private boolean createDeliveryTable(Long storeId) {
        StringBuffer buffer1 = new StringBuffer();
        buffer1.append("CREATE TABLE delivery_").append(storeId).append(" LIKE delivery_1;");
        jdbcTemplate.update(buffer1.toString());

        StringBuffer buffer2 = new StringBuffer();
        buffer2.append("CREATE TABLE delivery_status_").append(storeId).append(" LIKE delivery_status_1;");
        jdbcTemplate.update(buffer2.toString());
        return true;
    }

    private void updateRedis(Long storeId) {
        ExStore store = exStoreMapper.selectByPrimaryKey(storeId);
        if (store != null) {
            CacheTool cacheTool1 = CacheTool.initStore();
            cacheTool1.saveBean(String.valueOf(store.getId()), store);
        }
    }

    private Pager<AdminUser> listAdminUser(HttpServletRequest request, AdminUserListDto adminUserListDto) {
        adminUserListDto.initSort();
        adminUserListDto.initPage();
        HttpSession session = request.getSession();
        User user = JSON.parseObject(session.getAttribute(session.getId()).toString(), User.class);

        adminUserListDto.setStoreId(user.getStoreId());
        adminUserListDto.setAdminType(user.getAdminType());
        int count = userMapper.getAdminUserCount(adminUserListDto);
        Pager<AdminUser> pager = new Pager(count, adminUserListDto.getPageNumber(), adminUserListDto.getPageSize());
        if(count > 0) {
            pager.setList(userMapper.listAdminUser(adminUserListDto));
        }
        return pager;
    }

}
