package cn.bs.bsmanagebasic.service.impl;

import cn.bs.bsmanagebasic.common.enums.AdminTypeEnum;
import cn.bs.bsmanagebasic.common.util.Pager;
import cn.bs.bsmanagebasic.dao.jisu.StoreStaffMapper;
import cn.bs.bsmanagebasic.entity.User;
import cn.bs.bsmanagebasic.entity.dto.StoreStaffDto;
import cn.bs.bsmanagebasic.entity.filter.StoreStaffFilter;
import cn.bs.bsmanagebasic.service.StoreStaffService;
import cn.bs.bsmanagebasic.service.UserService;
import cn.bs.bsmanagecommon.exception.BusinessException;
import com.alibaba.fastjson.JSON;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * 门店外派员
 */
@Service
public class StoreStaffServiceImpl implements StoreStaffService {

    @Resource
    private StoreStaffMapper storeStaffMapper;

    @Resource
    private HttpServletRequest request;

    @Autowired
    private UserService userService;

    /**
     * 获取门店外派员信息
     * @return
     */
    @Override
    public List<StoreStaffDto> getStoreStaffList() {
//        Integer storeId = Integer.parseInt(request.getHeader("storeId"));
        HttpSession session = request.getSession();
        User user = JSON.parseObject(session.getAttribute(session.getId()).toString(), User.class);
        Long storeId = user.getStoreId();
        if (user.getAdminType() == AdminTypeEnum.SYSTEM_ADMIN.getType()){
            storeId = null;
        }
        return storeStaffMapper.getStoreStaffList(storeId);
    }


    @Override
    public Pager<StoreStaffDto> getStoreStaffPage(StoreStaffFilter filter) {
        filter.initPage();
        HttpSession session = request.getSession();
        User user = JSON.parseObject(session.getAttribute(session.getId()).toString(), User.class);
        Long storeId = user.getStoreId();
        filter.setStoreId(storeId);
        Pager<StoreStaffDto> pager = new Pager<>(listCount(filter), filter.getPageNumber(), filter.getPageSize());
        List<StoreStaffDto> list = storeStaffMapper.getStoreStaffPage(filter);
        pager.setList(list);
        return pager;
    }

    /**
     * 新增外派员
     * @param user
     * @return
     */
    @Override
    public boolean addStoreStaff(User user) {
        //判断用户是否存在
        User checkUser = userService.getByPrimaryKey(user.getUserId());
        if (checkUser != null){
            throw new BusinessException("用户已存在");
        }
        if (user.getUserId() == null || user.getUserId().isEmpty()){
            throw new BusinessException("账号不能为空");
        }
        user.setPassword("e10adc3949ba59abbe56e057f20f883e");
        user.setCreatetime(new Date());
        user.setStatus(1);
        user.setAdminType(AdminTypeEnum.DELIVERER.getType());
        HttpSession session = request.getSession();
        User user1 = JSON.parseObject(session.getAttribute(session.getId()).toString(), User.class);
        Long storeId = user1.getStoreId();
        user.setCodeType(1);
        user.setContainsShelf(1);
        user.setStoreId(storeId);
        user.setSmsPrice(BigDecimal.valueOf(0.001));
        user.setSmsBalance(BigDecimal.valueOf(100.000));
        if (user.getAdminType() == AdminTypeEnum.SYSTEM_ADMIN.getType()){
            return false;
        }
        return storeStaffMapper.addStoreStaff(user);
    }

    /**
     * 修改外派员信息
     * @param storeStaff
     * @return
     */
    @Override
    public boolean updateStoreStaffById(User storeStaff) {
        HttpSession session = request.getSession();
        User user = JSON.parseObject(session.getAttribute(session.getId()).toString(), User.class);
        Long storeId = user.getStoreId();
        storeStaff.setStoreId(storeId);
        if (user.getAdminType() == AdminTypeEnum.SYSTEM_ADMIN.getType()){
            return false;
        }
        return storeStaffMapper.updateStoreStaffById(storeStaff);
    }

    /**
     * 删除
     * @param userId
     * @return
     */
    @Override
    public boolean deleteStoreStaffById(String userId) {
        return storeStaffMapper.deleteStoreStaffById(userId);
    }


    /**
     * 查询
     * @param storeStaff
     * @return
     */
    @Override
    public List<StoreStaffDto> queryStoreStaff(StoreStaffFilter storeStaff) {
        HttpSession session = request.getSession();
        User user = JSON.parseObject(session.getAttribute(session.getId()).toString(), User.class);
        Long storeId = user.getStoreId();
        if (user.getAdminType() != AdminTypeEnum.SYSTEM_ADMIN.getType()){
            storeStaff.setStoreId(storeId);
        }
        return storeStaffMapper.queryStoreStaff(storeStaff);
    }

    /**
     * 条数
     * @param filter
     * @return
     */
    @Override
    public Integer listCount(StoreStaffFilter filter) {
        HttpSession session = request.getSession();
        User user = JSON.parseObject(session.getAttribute(session.getId()).toString(), User.class);
        Long storeId = user.getStoreId();
        filter.setStoreId(storeId);
        return storeStaffMapper.listCount(filter);
    }

    /**
     * 详情
     * @param userId
     * @return
     */
    @Override
    public StoreStaffDto detailStaff(String userId) {
        return storeStaffMapper.detailStaff(userId);
    }
}
