package cn.bs.bsmanagebasic.service.impl;

import cn.bs.bsmanagebasic.common.util.CacheTool;
import cn.bs.bsmanagebasic.dao.jeesite.StoreTemplateMapper;
import cn.bs.bsmanagebasic.dao.jisu.StoreDeliveryMsgTemplateMapper;
import cn.bs.bsmanagebasic.entity.StoreDeliveryMsgTemplate;
import cn.bs.bsmanagebasic.entity.StoreDeliveryMsgTemplateExample;
import cn.bs.bsmanagebasic.entity.StoreTemplate;
import cn.bs.bsmanagebasic.entity.StoreTemplateExample;
import cn.bs.bsmanagebasic.entity.dto.StoreTemplateDto;
import cn.bs.bsmanagebasic.service.StoreTemplateService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author sunjj
 * @version 1.0.0
 * @ClassName StoreTemplateServiceImpl.java
 * @Description 短信模板处理
 * @createTime 2019年07月23日 11:30:00
 */
@Service
public class StoreTemplateServiceImpl implements StoreTemplateService {
    @Autowired
    StoreDeliveryMsgTemplateMapper mapper;
    @Autowired
    StoreTemplateMapper storeTemplateMapper;

    @Override
    public List<StoreDeliveryMsgTemplate> getTemplates() {
        StoreDeliveryMsgTemplateExample example = new StoreDeliveryMsgTemplateExample();
        StoreDeliveryMsgTemplateExample.Criteria criteria = example.createCriteria();
        criteria.andIsdelEqualTo(0);
        return mapper.selectByExample(example);
    }

    @Override
    public int saveStoreTemplate(StoreTemplate storeTemplate) {
        storeTemplate.setIsUsed(Long.valueOf(1));
        return storeTemplateMapper.insertSelective(storeTemplate);
    }

    @Override
    public boolean updateByStoreId(StoreTemplate storeTemplate) {
        StoreTemplateExample example = new StoreTemplateExample();
        StoreTemplateExample.Criteria criteria = example.createCriteria();
        criteria.andStoreIdEqualTo(storeTemplate.getStoreId());
        storeTemplateMapper.updateByExampleSelective(storeTemplate, example);
        return true;
    }

    /**
     * @description 编辑短信模板配置
     * @author sunjj
     */
    @Override
    public void updateTemplateByStoreId(Long storeId, Long templateId) {
        if (templateId == null) {
            // 如果templatedId为空，禁用所有短信模板
            StoreTemplate storeTemplate = new StoreTemplate();
            storeTemplate.setStoreId(storeId);
            storeTemplate.setIsUsed(0L);
            this.updateByStoreId(storeTemplate);
        } else {
            // 根据storeId 和 templatedId 查询符合条件的短信模板配置
            StoreTemplateExample example = new StoreTemplateExample();
            StoreTemplateExample.Criteria criteria = example.createCriteria();
            criteria.andStoreIdEqualTo(storeId);
            criteria.andTemplateIdEqualTo(templateId);
            List<StoreTemplate> storeTemplates = storeTemplateMapper.selectByExample(example);

            StoreTemplate storeTemplate = new StoreTemplate();
            storeTemplate.setStoreId(storeId);
            storeTemplate.setTemplateId(templateId);


            if (null == storeTemplates || storeTemplates.size() == 0) {
                // 如果没有配置，插入一条数据，默认为不启用的状态
                storeTemplate.setIsUsed(0L);
                this.saveStoreTemplate(storeTemplate);
            } else if (null != storeTemplates.get(0).getIsUsed() && 1L == storeTemplates.get(0).getIsUsed()) {
                // 如果查询到的配置为已使用状态，不做处理
                return;
            }

            // 更换短信模板配置
            storeTemplateMapper.updateTemplateByStoreId(storeTemplate);

            // 更新缓存
            List<StoreTemplateDto> templates = mapper.getTemplateByStoreId(storeId);
            if (templates == null) {
                templates = new ArrayList<>();
            }

            CacheTool cacheTool = CacheTool.initStoreModel();
            cacheTool.saveBean(String.valueOf(storeId), templates);
        }
    }

    @Override
    public int saveByDefaultTemplate(StoreTemplate storeTemplate) {
        List<Long> defaultTemplateIds = getDefaultTemplateIds();
        Long templateId = storeTemplate.getTemplateId();
        Long storeId = storeTemplate.getStoreId();
        boolean match = defaultTemplateIds.stream().anyMatch(id -> id.compareTo(templateId) == 0);
        if (!match) {
            defaultTemplateIds.add(templateId);
        }

        List<StoreTemplate> storeTemplates = defaultTemplateIds.stream().map(item -> {
            StoreTemplate template = new StoreTemplate();
            template.setStoreId(storeId);
            template.setTemplateId(item);

            if (item.compareTo(templateId) == 0) {
                template.setIsUsed(1L);
            } else {
                template.setIsUsed(0L);
            }

            return template;
        }).collect(Collectors.toList());

        return storeTemplateMapper.insertList(storeTemplates);
    }

    private List<Long> getDefaultTemplateIds() {
        List<Long> templateIds = new ArrayList<>();
        templateIds.add(2L);
        templateIds.add(3L);
        templateIds.add(4L);
        templateIds.add(5L);
        return templateIds;
    }
}
