package cn.bs.bsmanagebasic.service.impl;

import cn.bs.bsmanagebasic.common.util.CacheTool;
import cn.bs.bsmanagebasic.dao.jeesite.SystemMapper;
import cn.bs.bsmanagebasic.entity.filter.SystemFilter;
import cn.bs.bsmanagebasic.entity.sys.SysParam;
import cn.bs.bsmanagebasic.entity.sys.SysType;
import cn.bs.bsmanagebasic.service.SystemService;
import cn.bs.bsmanagecommon.exception.BusinessException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * 〈功能概述〉<br>
 *
 * @className: SystemServiceImpl
 * @package: cn.bs.bsmanagebasic.service.impl
 * @author: kael
 * @date: 2019/12/2 0002 上午 10:56
 */
@Service
public class SystemServiceImpl implements SystemService {

    @Autowired
    private SystemMapper systemMapper;

    @Override
    public List<SysParam> getDataByType(Integer type){
        if(type == null){
            throw new BusinessException("传入参数不能为空");
        }
        CacheTool cacheTool = CacheTool.initSysParamTool();
        List<SysParam> sysParams = cacheTool.getBean(type.toString(),SysParam.class);
        if(sysParams == null){
            SystemFilter filter = new SystemFilter();
            filter.setType(type);
            sysParams = systemMapper.getDataByType(filter);
            cacheTool.saveBean(type.toString(), sysParams);
        }
        return sysParams;
    }

    @Override
    public List<SysParam> getExpressAndPlatform(){
        List<SysParam> sysParams = new ArrayList<>();
        sysParams.addAll(getDataByType(1));
        sysParams.addAll(getDataByType(2));
        return sysParams;
    }

    @Override
    public List<SysType> getAllListType(){
        CacheTool cacheTool = CacheTool.initSysTypeTool();
        List<SysType> sysTypes = cacheTool.getBean("0",SysType.class);
        if(sysTypes == null){
            sysTypes = systemMapper.getAllListType();
            cacheTool.saveBean("0", sysTypes);
        }
        return sysTypes;
    }

    @Override
    public boolean typeEnable(Integer id){
        if(id == null){
            throw new BusinessException("传入参数不能为空");
        }
        boolean fale = systemMapper.enableById(id) > 0;
        if(fale){
            CacheTool cacheTool = CacheTool.initSysTypeTool();
            cacheTool.saveBean("0", systemMapper.getAllListType());
        }
        return fale;
    }

    @Override
    public boolean saveType(SysType sysType){
        boolean fale = false;
        SystemFilter filter = new SystemFilter();
        filter.setParamName(sysType.getParamName());
        if(sysType.getId() == null){
            fale = systemMapper.addType(sysType) > 0;
        }else{
            fale = systemMapper.updateType(sysType) > 0;
        }
        if(fale){
            CacheTool cacheTool = CacheTool.initSysTypeTool();
            cacheTool.saveBean("0", systemMapper.getAllListType());
        }
        return fale;
    }

    @Override
    public boolean deleteType(Integer id){
        boolean fale = systemMapper.deleteType(id) > 0;
        if(fale){
            CacheTool cacheTool = CacheTool.initSysTypeTool();
            cacheTool.saveBean("0", systemMapper.getAllListType());
        }
        return fale;
    }

    @Override
    public boolean saveParam(SysParam sysParam){
        boolean fale = false;
        if(sysParam.getTypeId()==null){
            throw new BusinessException("传入类型不能为空");
        }
        if(sysParam.getId() == null){
            sysParam.setIsDisable(false);
            fale = systemMapper.addParam(sysParam) > 0;
        }else{
            fale = systemMapper.updateParam(sysParam) > 0;
        }
        if(fale){
            SystemFilter filter = new SystemFilter();
            filter.setType(sysParam.getTypeId());
            CacheTool cacheTool = CacheTool.initSysParamTool();
            cacheTool.saveBean(sysParam.getTypeId().toString(), systemMapper.getDataByType(filter));
        }
        return fale;
    }

    @Override
    public boolean deleteParam(Integer id){
        SystemFilter filter = new SystemFilter();
        filter.setId(id);
        List<SysParam> sysParams = systemMapper.getDataByType(filter);
        if(sysParams==null||sysParams.size()==0){
            throw new BusinessException("删除失败，不存在该记录");
        }
        boolean fale = systemMapper.deleteParam(id) > 0;
        if(fale){
            filter.setId(null);
            filter.setType(sysParams.get(0).getTypeId());
            CacheTool cacheTool = CacheTool.initSysParamTool();
            cacheTool.saveBean(sysParams.get(0).getTypeId().toString(),systemMapper.getDataByType(filter));
        }
        return fale;
    }

    @Override
    public boolean enableParam(Integer id,Integer typeId){
        if(id == null){
            throw new BusinessException("传入参数不能为空");
        }
        boolean fale = systemMapper.enableParamById(id) > 0;
        if(fale){
            SystemFilter filter = new SystemFilter();
            filter.setType(typeId);
            CacheTool cacheTool = CacheTool.initSysParamTool();
            cacheTool.saveBean(typeId.toString(),systemMapper.getDataByType(filter));
        }
        return fale;
    }

}
