package cn.bs.bsmanagebasic.service.impl;

import cn.bs.bsmanagebasic.common.util.CacheTool;
import cn.bs.bsmanagebasic.dao.jeesite.ExStoreMapper;
import cn.bs.bsmanagebasic.dao.jisu.RoleMapper;
import cn.bs.bsmanagebasic.dao.jisu.UserMapper;
import cn.bs.bsmanagebasic.entity.ExStore;
import cn.bs.bsmanagebasic.entity.User;
import cn.bs.bsmanagebasic.entity.UserExample;
import cn.bs.bsmanagebasic.entity.filter.UserFilter;
import cn.bs.bsmanagebasic.entity.user.LoginData;
import cn.bs.bsmanagebasic.service.UserService;
import cn.bs.bsmanagebasic.utils.Md5Util;
import cn.bs.bsmanagecommon.exception.BusinessException;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpSession;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author sunjj
 * @version 1.0.0
 * @ClassName UserServiceImpl.java
 * @Description TODO
 * @createTime 2019年07月24日 17:31:00
 */
@Service
@Slf4j
public class UserServiceImpl implements UserService {
    @Autowired
    private UserMapper mapper;

    @Autowired
    private ExStoreMapper exStoreMapper;

    @Resource
    private RoleMapper roleMapper;

    @Override
    public User getByPrimaryKey(String userId) {
        return mapper.selectByPrimaryKey(userId);
    }

    @Override
    public List<User> getByStoreId(Long storeId) {
        UserExample example = new UserExample();
        UserExample.Criteria criteria = example.createCriteria();
        criteria.andStoreIdEqualTo(storeId);
        return mapper.selectByExample(example);
    }

    @Override
    public Integer save(User user) {
        return mapper.insertSelective(user);
    }

    @Override
    public Integer updateByStoreId(User user) {
        UserExample example = new UserExample();
        UserExample.Criteria criteria = example.createCriteria();
        criteria.andStoreIdEqualTo(user.getStoreId());
        int update = mapper.updateByExampleSelective(user, example);
        // 更新缓存
        List<User> users = mapper.selectByExample(example);
        if (null != users && users.size() > 0) {
            CacheTool cacheToolUser = CacheTool.initUser();
            cacheToolUser.saveBean(users.get(0).getUserId(), users.get(0));
        }
        return update;
    }

    @Override
    public LoginData login(UserFilter filter){
        User user = this.getByPrimaryKey(filter.getUserId());
        if(user == null){
            throw new BusinessException("用户不存在");
        }
        String md5Pwd = Md5Util.md5AsLowerHex(filter.getPassword());
        if(!user.getPassword().equals(md5Pwd)){
            throw new BusinessException("密码错误，请重试");
        }
        if (user.getStoreId() != 0) {
            if(user.getStatus() == 0) {
                throw new BusinessException("账号被禁用");
            }
            ExStore store = exStoreMapper.selectStoreById(user.getStoreId().longValue());
            if(store.getStatus() == 0) {
                throw new BusinessException("账号被禁用");
            }
            //未审核
            if(store.getAuditStatus() == 0) {
                throw new BusinessException("未审核");
            }
            //审核未通过
            if(store.getAuditStatus() == 2) {
                throw new BusinessException("审核未通过");
            }
        }
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpSession session= servletRequestAttributes.getRequest().getSession();
        session.setAttribute(session.getId(), JSON.toJSONString(user));
        return LoginData.builder()
                .userId(user.getUserId())
                .userName(user.getUserName())
                .password(md5Pwd)
                .phoneLogo(user.getPhoneLogo())
//                .shortAddress(store.getShortAddress())
//                .detailAddress(store.getAddress())
//                .tellPhone(store.getTakeExpressPhone())
//                .startTime(store.getStartTime())
//                .endTime(store.getEndTime())
//                .shortName(store.getShortName())
                .email(user.getUsermail())
                .status(user.getStatus())
                .smsPrice(user.getSmsPrice())
                .codeType(user.getCodeType())
                .storeId(user.getStoreId())
                .token(session.getId())
                .build();
    }
    @Override
    public Map getRole(){
        User user = getUser();
        Map param = new HashMap();
        //获取用户角色
        /*List<String> roles = roleMapper.getRole(user.getUserId());*/
        CacheTool cacheTool = CacheTool.initUserAvatarTool();
        String userAvatar = (String)cacheTool.getBean(user.getUserId());
        userAvatar = Strings.isBlank(userAvatar)?user.getPhoneLogo():userAvatar;
        param.put("name",user.getUserName());
        param.put("avatar",userAvatar);
        param.put("roles","");
        param.put("introduction",user.getSmsPrice());
        param.put("storeId",user.getStoreId());
        param.put("user",user);
        return param;
    }

    @Override
    public Boolean logout(){
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpSession session= servletRequestAttributes.getRequest().getSession();
        session.removeAttribute(session.getId());
        return true;
    }

    @Override
    public Boolean updateUserAvatar(UserFilter filter){
        if(Strings.isBlank(filter.getUserId())){
            throw new BusinessException("传入参数不能为空");
        }
        CacheTool cacheTool = CacheTool.initUserAvatarTool();
        cacheTool.saveBean(filter.getUserId(),filter.getImage());
        return true;
    }

    @Override
    public Boolean updateUserPassword(UserFilter filter){
        if(Strings.isBlank(filter.getUserId())||Strings.isBlank(filter.getPassword())||Strings.isBlank(filter.getNewPassword())){
            throw new BusinessException("传入参数不能为空");
        }
        User user = this.getByPrimaryKey(filter.getUserId());
        String md5Pwd = Md5Util.md5AsLowerHex(filter.getPassword());
        if(!user.getPassword().equals(md5Pwd)){
            throw new BusinessException("密码错误，请重试");
        }
        User newUser = new User();
        newUser.setUserId(filter.getUserId());
        newUser.setPassword(Md5Util.md5AsLowerHex(filter.getNewPassword()));
        return mapper.updateByPrimaryKeySelective(newUser)>0;
    }

    private User getUser() {
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpSession session= servletRequestAttributes.getRequest().getSession();
        return JSON.parseObject(session.getAttribute(session.getId()).toString(),User.class);
    }


    @Override
    public List<User> selectByExample(UserExample example) {
        return mapper.selectByExample(example);
    }
}
