package cn.bs.bsmanagecommon.util;

import org.joda.time.DateTime;
import org.joda.time.LocalDate;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.chrono.ChronoLocalDateTime;
import java.util.*;

/**
 * @author sunjj
 * @version 1.0.0
 * @ClassName DateUtil.java
 * @Description TODO
 * @createTime 2019年07月24日 17:00:00
 */
public class DateUtil {
    private static final Map<String, ThreadLocal<SimpleDateFormat>> dateFormatPool = new HashMap<String, ThreadLocal<SimpleDateFormat>>();

    public static final String TIME_PATTERN = "HH:mm:ss";
    public static final String DATE_TIME_PATTERN = "yy-MM-dd HH:mm:ss";
    public static final String DATE_PATTERN = "yy-MM-dd";
    private static final Object dateFormatLock = new Object();

    public static String dealDateFormat(String oldDate, String pattern) {
        Date date1 = null;
        DateFormat df2 = null;
        try {
            oldDate = oldDate.replace("Z", " UTC");
            DateFormat df = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS Z");
            Date date = df.parse(oldDate);
            SimpleDateFormat df1 = new SimpleDateFormat("EEE MMM dd HH:mm:ss Z yyyy", Locale.UK);
            date1 = df1.parse(date.toString());
            df2 = new SimpleDateFormat(pattern);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return df2.format(date1);
    }

    public static Date toDate(String date) {
        try {
            return getDateFormat().parse(date);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return null;
    }

    public static String formatDate() {
        return getDateFormat().format(new Date());
    }

    public static String formatDate(Date date) {
        return getDateFormat().format(date);
    }

    public static Date addHour(int offsetHour) {
        return new DateTime().plusHours(offsetHour).toDate();
    }

    public static String addDay(String day, String pattern, int offsetDay) throws ParseException {
        SimpleDateFormat sdf = new SimpleDateFormat(pattern);
        Date date = getDateFormat().parse(day);
        return new DateTime(date).plusDays(offsetDay).toString(pattern);
    }

    public static String addDay(int offsetDay, String pattern) {
        return new DateTime().plusDays(offsetDay).toString(pattern);
    }

    public static String addMonth(String day, String pattern, int offsetMonth) throws ParseException {
        SimpleDateFormat sdf = new SimpleDateFormat(pattern);
        Date date = sdf.parse(day);
        return new DateTime(date).plusMonths(offsetMonth).toString(pattern);
    }

    public static String addMonth(int offsetMonth, String pattern) {
        return new DateTime().plusMonths(offsetMonth).toString(pattern);
    }

    /**
     * @param date 要与当前时间比较的日期
     * @return
     * @title: dateCompare
     * @description: 与当前时间比较日期大小
     */
    public static int compareNow(Date date) {
        return LocalDateTime.now().compareTo(asLocalDateTime(date));
    }

    public static Date getFirstDayOfMonth() {
        java.time.LocalDate now = java.time.LocalDate.now();
        return asDate(java.time.LocalDate.of(now.getYear(), now.getMonth(), 1));
    }

    public static Date addDay(int offsetDay) {
        return asDate(java.time.LocalDate.now().plusDays(offsetDay));
    }

    public static Date addDay(Date date, int offsetDay) {
        return asDate(addDayAsLocalDateTime(date, offsetDay));
    }

    public static LocalDateTime addDayAsLocalDateTime(Date date, int offsetDay) {
        return asLocalDateTime(date).plusDays(offsetDay);
    }

    public static Date asDate(java.time.LocalDate localDate) {
        return Date.from(localDate.atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
    }

    public static Date asDate(java.time.LocalDateTime localDateTime) {
        return Date.from(localDateTime.atZone(ZoneId.systemDefault()).toInstant());
    }

    public static java.time.LocalDate asLocalDate(Date date) {
        return Instant.ofEpochMilli(date.getTime()).atZone(ZoneId.systemDefault()).toLocalDate();
    }

    public static java.time.LocalDateTime asLocalDateTime(Date date) {
        return Instant.ofEpochMilli(date.getTime()).atZone(ZoneId.systemDefault()).toLocalDateTime();
    }

    private static SimpleDateFormat getDateFormat() {
        ThreadLocal<SimpleDateFormat> tl = dateFormatPool.get(DATE_PATTERN);
        if (null == tl) {
            synchronized (dateFormatLock) {
                tl = dateFormatPool.get(DATE_PATTERN);
                if (null == tl) {
                    tl = new ThreadLocal<SimpleDateFormat>() {
                        @Override
                        protected synchronized SimpleDateFormat initialValue() {
                            return new SimpleDateFormat(DATE_PATTERN);
                        }
                    };
                    dateFormatPool.put(DATE_PATTERN, tl);
                }
            }
        }
        return tl.get();
    }
}
